@@ -30,25 +30,16 @@ package Sidef::Deparse::Perl {
                               },
 
             lazy_ops => {
-                '?'  => '?',
-                '||' => '||',
-                '&&' => '&&',
-                ':=' => '//=',
-
-                # '='     => '=',
-                '||='   => '||=',
-                '&&='   => '&&=',
-                '\\\\'  => '//',
-                '\\\\=' => '//=',
+                         '?'     => '?',
+                         '||'    => '||',
+                         '&&'    => '&&',
+                         ':='    => '//=',
+                         '||='   => '||=',
+                         '&&='   => '&&=',
+                         '\\\\'  => '//',
+                         '\\\\=' => '//=',
                         },
 
-            #~ overload_methods => {
-            #~ to_str  => q{""},
-            #~ to_s    => q{""},
-            #~ to_bool => q{bool},
-            #~ to_num  => q{0+},
-            #~ },
-
             overload_methods => {
                                  '=='  => 'eq',
                                  '<=>' => 'cmp',
@@ -60,13 +51,13 @@ package Sidef::Deparse::Perl {
                   Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
                   Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
                   Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
-                  Sidef::DataTypes::Array::Enumerator     Sidef::Types::Array::Enumerator
                   Sidef::DataTypes::Array::MultiArray     Sidef::Types::Array::MultiArray
                   Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
                   Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
                   Sidef::DataTypes::String::String        Sidef::Types::String::String
                   Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
                   Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
+                  Sidef::DataTypes::Range::Range          Sidef::Types::Range::Range
                   Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
                   Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
                   Sidef::DataTypes::Block::Block          Sidef::Types::Block::Block
@@ -81,6 +72,7 @@ package Sidef::Deparse::Perl {
                   Sidef::DataTypes::Sidef::Sidef          Sidef
                   Sidef::DataTypes::Object::Lazy          Sidef::Object::Lazy
                   Sidef::DataTypes::Object::LazyMethod    Sidef::Object::LazyMethod
+                  Sidef::DataTypes::Object::Enumerator    Sidef::Object::Enumerator
                   )
             },
 
@@ -165,6 +157,15 @@ HEADER
         }
     }
 
+    sub load_mod {
+        my ($self, $mod) = @_;
+        $self->top_add(
+                       $self->{opt}{c}
+                       ? qq{main::__load_sidef_module__("$mod");}
+                       : "use $mod;"
+                      );
+    }
+
     sub _get_reftype {
         my ($self, $obj) = @_;
 
@@ -173,9 +174,8 @@ HEADER
         if (exists $self->{data_types}{$ref}) {
             my $target = $self->{data_types}{$ref};
 
-            state $cache = {};
-            $cache->{$ref} //= do {
-                $self->top_add("use $target;");
+            $self->{_reftype_cache}{$ref} //= do {
+                $self->load_mod($target);
                 1;
             };
 
@@ -387,63 +387,29 @@ HEADER
 
     sub _dump_indices {
         my ($self, $array) = @_;
-        '[' . join(
+
+        join(
             ',',
-            map {
+            grep { $_ ne '' } map {
                     ref($_) eq 'Sidef::Types::Number::Number' ? $_->_get_double
                   : ref($_) ? ($self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}))
                   : $_
               } @{$array}
-          )
-          . ']';
-    }
-
-    sub _dump_unpacked_indices {
-        my ($self, $array) = @_;
-        '[' . join(
-            ',',
-            map {
-                '@{'
-                  . (
-                     ref($_)
-                     ? ($self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}))
-                     : die "[ERROR] Value '$_' can't be unpacked in Array index!"
-                    )
-                  . '}'
-              } @{$array}
-          )
-          . ']';
+            );
     }
 
     sub _dump_lookups {
         my ($self, $array) = @_;
-        '{' . join(
+
+        join(
             ',',
-            map {
+            grep { $_ ne '' } map {
                 (ref($_) eq 'Sidef::Types::String::String' or ref($_) eq 'Sidef::Types::Number::Number')
                   ? $self->_dump_string($_->get_value)
                   : ref($_) ? ($self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}))
-                  : $_
+                  : qq{"\Q$_\E"}
               } @{$array}
-          )
-          . '}';
-    }
-
-    sub _dump_unpacked_lookups {
-        my ($self, $array) = @_;
-        '{' . join(
-            ',',
-            map {
-                '@{'
-                  . (
-                     ref($_)
-                     ? ($self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}))
-                     : die "[ERROR] Value '$_' can't be unpacked in Hash lookup!"
-                    )
-                  . '}'
-              } @{$array}
-          )
-          . '}';
+            );
     }
 
     sub _dump_var_attr {
@@ -551,7 +517,7 @@ HEADER
             $code = join(',', exists($obj->{self}) ? $self->deparse_expr($obj) : $self->deparse_script($obj));
         }
         elsif ($ref eq 'Sidef::Variable::Variable') {
-            if ($obj->{type} eq 'var') {
+            if ($obj->{type} eq 'var' or $obj->{type} eq 'has') {
 
                 my $name = $obj->{name} . $refaddr;
 
@@ -589,26 +555,47 @@ HEADER
                     }
 
                     # The name of the function
-                    $code .= "\$$obj->{name}$refaddr=";
+                    $code .= "\$$obj->{name}$refaddr";
 
                     # Deparse the block of the method/function
                     {
                         local $self->{function} = refaddr($block);
                         local $self->{parent_name} = [$obj->{type}, "$obj->{class}::$name"];
                         push @{$self->{function_declarations}}, [$self->{function}, "my \$$obj->{name}$refaddr;"];
-                        $code .= $self->deparse_expr({self => $block});
+
+                        if ($self->{ref_class}) {
+                            push @{$self->{function_declarations}},
+                              [ $self->{function},
+                                qq{state \$$obj->{name}_code$refaddr = UNIVERSAL::can("\Q$self->{class_name}\E", "\Q$name\E");}
+                              ];
+                        }
+
+                        if ((my $content = $self->deparse_expr({self => $block})) ne '') {
+                            $code .= "=$content";
+                        }
                     }
 
                     # Check to see if the method/function has kids (can do multiple dispatch)
                     if (exists $obj->{value}{kids}) {
                         chop $code;
-
                         my @kids = map {
                             local $_->{type} = 'func';
                             'do{' . $self->deparse_expr({self => $_}) . '}';
                         } @{$obj->{value}{kids}};
 
-                        $code .= ',kids=>[' . join(',', @kids) . '])';
+                        $code .= ',kids=>[' . join(',', @kids);
+
+                        if ($self->{ref_class}) {
+                            $code .= qq{,(defined(\$$obj->{name}_code$refaddr)?}
+                              . qq{Sidef::Types::Block::Block->new(code=>\$$obj->{name}_code$refaddr):())};
+                        }
+
+                        $code .= '])';
+                    }
+                    elsif ($self->{ref_class}) {
+                        chop $code;
+                        $code .= qq{,(defined(\$$obj->{name}_code$refaddr)?(kids=>[}
+                          . qq{Sidef::Types::Block::Block->new(code=>\$$obj->{name}_code$refaddr)]):()))};
                     }
 
                     # Check the return value (when "-> Type" is specified)
@@ -666,14 +653,21 @@ HEADER
                 }
             }
         }
-        elsif ($ref eq 'Sidef::Object::Unary') {
+        elsif ($ref eq 'Sidef::Operator::Unary') {
             ## OK
         }
         elsif ($ref eq 'Sidef::Variable::Local') {
             $code = 'local ' . $self->deparse_script($obj->{expr});
         }
         elsif ($ref eq 'Sidef::Variable::Global') {
-            $code = '$' . $obj->{class} . '::' . $obj->{name};
+            my $name = '$' . $obj->{class} . '::' . $obj->{name};
+            if (not exists($obj->{inited}) and defined($obj->{expr})) {
+                $obj->{inited} = 1;
+                $code = $name . '=do{' . $self->deparse_script($obj->{expr}) . '}';
+            }
+            else {
+                $code = $name;
+            }
         }
         elsif ($ref eq 'Sidef::Variable::ClassVar') {
             $code = '$' . $self->_get_reftype($obj->{class}) . '::' . $obj->{name};
@@ -723,7 +717,7 @@ HEADER
             }
         }
         elsif ($ref eq 'Sidef::Variable::ConstInit') {
-            $code = join(';', map { $self->deparse_expr({self => $_}) } @{$obj->{vars}});
+            $code = join(($obj->{type} eq 'global' ? ',' : ';'), map { $self->deparse_expr({self => $_}) } @{$obj->{vars}});
         }
         elsif ($ref eq 'Sidef::Variable::Init') {
             $code = $self->_dump_init_vars($obj);
@@ -841,7 +835,8 @@ HEADER
                                 $code .= qq{"\Q$var->{name}\E"=>} . $self->_dump_var($var) . ', ';
                             }
 
-                            $code .= '},__PACKAGE__;' . 'if(defined(my$sub=$self->can("init"))){$sub->($self)}' . '$self;';
+                            $code .=
+                              '},__PACKAGE__;' . 'if(defined(my$sub=UNIVERSAL::can($self,"init"))){$sub->($self)}' . '$self;';
 
                             $code .=
                                 '},'
@@ -1044,24 +1039,8 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Block::ForIn') {
 
             my $expr = $self->deparse_expr({self => $obj->{expr}});
-
             local $obj->{block}{init_vars} = bless({vars => $obj->{vars}}, 'Sidef::Variable::Init');
-
-            ## Old way:
-            # my $block = $self->deparse_expr({self => $obj->{block}});
-
-            my $vars;
-            if (@{$obj->{block}{init_vars}{vars}}) {
-                $vars = $self->_dump_sub_init_vars(@{$obj->{block}{init_vars}{vars}});
-            }
-
-            my $block = $self->deparse_bare_block($obj->{block}{code});
-
-            $block = (
-                      defined($vars)
-                      ? ('sub { ' . $vars . 'do ' . $block . '}')
-                      : ('sub ' . $block)
-                     );
+            my $block = $self->deparse_expr({self => $obj->{block}});
 
             if (
                 @{$obj->{vars}} > 1
@@ -1073,9 +1052,10 @@ HEADER
                     'do{my $obj='
                   . $expr . ';'
                   . 'my $block='
-                  . $block . ';'
-                  . 'for my$group(ref($obj)?$obj->SUPER::isa("ARRAY")?@$obj:@{$obj->to_a}:()){'
-                  . '$block->((ref($group)&&$group->SUPER::isa("ARRAY")?@$group:$group))}}';
+                  . $block
+                  . '->{code};'
+                  . 'for my$group(ref($obj)?UNIVERSAL::isa($obj,"ARRAY")?@$obj:@{$obj->to_a}:()){'
+                  . '$block->((ref($group)&&UNIVERSAL::isa($group,"ARRAY")?@$group:$group))}}';
             }
             else {
 
@@ -1083,11 +1063,12 @@ HEADER
                     'do{my$obj='
                   . $expr . ';'
                   . 'my$block='
-                  . $block . ';'
-                  . 'if(ref($obj)){if(defined(my$sub=$obj->SUPER::can("iter"))){my$iter=$sub->($obj);'
+                  . $block
+                  . '->{code};'
+                  . 'if(ref($obj)){if(defined(my$sub=UNIVERSAL::can($obj,"iter"))){my$iter=$sub->($obj);'
                   . 'while(defined(my$item'
                   . '=$iter->run)){$block->($item)}}'
-                  . 'else{for my$item($obj->SUPER::isa("ARRAY")?@$obj:@{$obj->to_a}){'
+                  . 'else{for my$item(UNIVERSAL::isa($obj,"ARRAY")?@$obj:@{$obj->to_a}){'
                   . '$block->($item)}}}}';
             }
         }
@@ -1110,7 +1091,7 @@ HEADER
                     map {
                         $self->_dump_string($_) . '=>'
                           . (defined($obj->{$_}) ? $self->deparse_expr({self => $obj->{$_}}) : 'undef')
-                      } keys(%{$obj})
+                      } sort keys(%{$obj})
                   )
                   . ')';
             }
@@ -1152,7 +1133,7 @@ HEADER
               . ' else { } }';
         }
         elsif ($ref eq 'Sidef::Types::Block::Gather') {
-            $self->top_add("use Sidef::Types::Array::Array;");
+            $self->load_mod("Sidef::Types::Array::Array");
             $code =
                 "do{my \@_$refaddr;"
               . $self->deparse_bare_block($obj->{block}->{code})
@@ -1241,7 +1222,7 @@ HEADER
         }
         elsif ($ref eq 'Sidef::Types::Number::Complex') {
 
-            #$code = $self->make_constant($ref, 'new', "Complex$refaddr", "'" . ${$obj}->Re . "'", "'" . ${$obj}->Im . "'");
+            # TODO: optimize this
             $code = $self->make_constant($ref, 'new', "Complex$refaddr",
                                          "'" . $obj->re->get_value . "'",
                                          "'" . $obj->im->get_value . "'");
@@ -1259,21 +1240,9 @@ HEADER
         elsif ($ref eq 'Sidef::Module::Func') {
             $code = $self->make_constant($ref, '__NEW__', "MOD_F$refaddr", $self->_dump_string($obj->{module}));
         }
-        elsif ($ref eq 'Sidef::Types::Range::RangeNumber') {
-            $code = $self->make_constant(
-                $ref,
-                '__new__',
-                "RangeNumber$refaddr",
-                map {
-                        "$_ => do{ my \$r = Math::GMPq::Rmpq_init(); Math::GMPq::Rmpq_set_str(\$r,'"
-                      . Math::GMPq::Rmpq_get_str($obj->{$_}, 10)
-                      . "', 10); \$r }"
-                  } keys(%{$obj})
-            );
-        }
-        elsif ($ref eq 'Sidef::Types::Range::RangeString') {
-            $code = $self->make_constant($ref, '__new__', "RangeString$refaddr",
-                                         map { "$_ => " . $self->_dump_string($obj->{$_}) } keys(%{$obj}));
+        elsif ($ref eq 'Sidef::Types::Range::RangeNumber' or $ref eq 'Sidef::Types::Range::RangeString') {
+            $code = $self->make_constant($ref, 'new', "Range$refaddr",
+                       map { 'Sidef::Types::Number::Number->_set_str(' . $obj->{$_}->_get_frac . ')' } ('from', 'to', 'step'));
         }
         elsif ($ref eq 'Sidef::Types::Glob::Backtick') {
             $code = $self->make_constant($ref, 'new', "Backtick$refaddr", $self->_dump_string(${$obj}));
@@ -1338,8 +1307,12 @@ HEADER
             $code = qq{CORE::die "Unimplemented at " . } . $self->_dump_string($obj->{file}) . qq{. " line $obj->{line}\\n"};
         }
         elsif (exists $self->{data_types}{$ref}) {
-            $self->top_add("use $self->{data_types}{$ref};");
-            $code = "'" . $self->{data_types}{$ref} . "'";
+            my $mod = $self->{data_types}{$ref};
+            $self->{_reftype_cache}{$mod} //= do {
+                $self->load_mod($mod);
+                1;
+            };
+            $code = "'" . $mod . "'";
         }
 
         # Array and hash indices
@@ -1349,25 +1322,33 @@ HEADER
 
                 my $ind = $expr->{ind}[$i];
                 if (exists $ind->{array}) {
+                    my $indices = $self->_dump_indices($ind->{array});
 
-                    my $pos = $ind->{array};
+                    $code = '@{' . $code . '}';
 
-                    if (substr($code, -1) eq '@') {
-                        $code .= $self->_dump_unpacked_indices($pos);
-                    }
-                    else {
-                        $code = '@{' . $code . '}' . $self->_dump_indices($pos);
+                    if ($indices ne '') {
+                        $code .= '['
+                          . 'map { ref($_) eq "Sidef::Types::Number::Number" ? Math::GMPq::Rmpq_get_d($$_) '
+                          . ': ref($_) eq "" ? $_ '
+                          . ': do { my $sub = UNIVERSAL::can($_, "..."); '
+                          . 'defined($sub) ? $sub->($_) : CORE::int($_) } } '
+                          . $indices . ']';
                     }
                 }
                 else {
+                    my $keys = $self->_dump_lookups($ind->{hash});
 
-                    my $key = $ind->{hash};
-
-                    if (substr($code, -1) eq '@') {
-                        $code .= $self->_dump_unpacked_lookups($key);
+                    if ($keys eq '') {
+                        $code = '%{' . $code . '}';
                     }
                     else {
-                        $code = '@{' . $code . '}' . $self->_dump_lookups($key);
+                        $code = '@{'
+                          . $code . '}' . '{'
+                          . 'map { ref($_) eq "Sidef::Types::String::String" ? $$_ '
+                          . ': ref($_) eq "" ? $_ '
+                          . ': do { my $sub = UNIVERSAL::can($_, "..."); '
+                          . 'defined($sub) ? $sub->($_) : "$_" } } '
+                          . $keys . '}';
                     }
                 }
 
@@ -1411,16 +1392,6 @@ HEADER
                     next;
                 }
 
-                # !!!Experimental!!!
-                #~ if ($ref eq 'Sidef::Types::Block::Break') {
-                #~ $code .= 'return Sidef::Types::Block::Break->new' . $self->deparse_args(@{$call->{arg}});
-                #~ next;
-                #~ }
-                #~ elsif ($ref eq 'Sidef::Types::Block::Next') {
-                #~ $code .= 'return Sidef::Types::Block::Next->new' . $self->deparse_args(@{$call->{arg}});
-                #~ next;
-                #~ }
-
                 if (defined $method) {
 
                     if ($ref eq 'Sidef::Variable::Ref') {    # variable refs
@@ -1509,7 +1480,7 @@ HEADER
                     }
 
                     # ! prefix-unary
-                    if ($ref eq 'Sidef::Object::Unary') {
+                    if ($ref eq 'Sidef::Operator::Unary') {
                         if ($method eq '!') {
                             $code = '(do{'
                               . $self->deparse_args(@{$call->{arg}})
@@ -1527,6 +1498,24 @@ HEADER
                             next;
                         }
 
+                        if ($method eq '@') {
+                            $code =
+                                '(do{ my $obj = '
+                              . $self->deparse_args(@{$call->{arg}})
+                              . '; my $sub = UNIVERSAL::can($obj, "to_a"); '
+                              . 'defined($sub) ? $sub->($obj) : Sidef::Types::Array::Array->new($obj) })';
+                            next;
+                        }
+
+                        if ($method eq '@|') {
+                            $code =
+                                '(do{ my $obj = '
+                              . $self->deparse_args(@{$call->{arg}})
+                              . '; my $sub = UNIVERSAL::can($obj, "..."); '
+                              . 'defined($sub) ? $sub->($obj) : $obj })';
+                            next;
+                        }
+
                         if ($method eq '~') {
                             $code = $self->deparse_args(@{$call->{arg}}) . '->not';
                             next;
@@ -1589,22 +1578,11 @@ HEADER
                         }
                     }
                     else {
+                        ## Old way:
+                        #$code .= '->${\\' . q{'} . $method . q{'} . '}';
 
-                        # Postfix dereference method
-                        if ($method eq '@' or $method eq '@*') {
-                            $self->top_add(qq{use experimental 'postderef';});
-                            $code .= '->' . $method;
-                        }
-
-                        # Operator-like method call
-                        else {
-
-                            ## Old way:
-                            #$code .= '->${\\' . q{'} . $method . q{'} . '}';
-
-                            ## New way:
-                            $code .= $self->_dump_op_call($method);
-                        }
+                        ## New way:
+                        $code .= $self->_dump_op_call($method);
                     }
                 }
 
