@@ -1,5 +1,6 @@
 package Sidef::Module::Func {
 
+    use 5.014;
     our $AUTOLOAD;
 
     sub __NEW__ {
@@ -17,20 +18,14 @@ package Sidef::Module::Func {
         my ($func) = ($AUTOLOAD =~ /^.*[^:]::(.*)$/);
 
         my @args = (
-            @arg
-            ? (
-               map {
-                   local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                   index(ref($_), 'Sidef::') == 0
-                     ? $_->get_value
-                     : $_
-                 } @arg
-              )
-            : ()
-        );
+                    @arg
+                    ? (map { index(ref($_), 'Sidef::') == 0 ? $_->get_value : $_ } @arg)
+                    : ()
+                   );
 
         my @results = do {
             local *UNIVERSAL::AUTOLOAD;
+            no strict 'refs';
             ($self->{module} . '::' . $func)->(@args);
         };
 
