@@ -9,7 +9,7 @@ package Sidef::Object::Object {
     use overload
       q{~~}   => \&{__PACKAGE__ . '::' . '~~'},
       q{bool} => sub {
-        if (defined(my $sub = $_[0]->can('to_b'))) {
+        if (defined(my $sub = UNIVERSAL::can($_[0], 'to_b'))) {
             $sub->($_[0]);
         }
         else {
@@ -17,7 +17,7 @@ package Sidef::Object::Object {
         }
       },
       q{0+} => sub {
-        if (defined(my $sub = $_[0]->can('to_n'))) {
+        if (defined(my $sub = UNIVERSAL::can($_[0], 'to_n'))) {
             $sub->($_[0]);
         }
         else {
@@ -25,7 +25,7 @@ package Sidef::Object::Object {
         }
       },
       q{""} => sub {
-        if (defined(my $sub = $_[0]->can('to_s'))) {
+        if (defined(my $sub = UNIVERSAL::can($_[0], 'to_s'))) {
             $sub->($_[0]);
         }
         else {
@@ -39,9 +39,9 @@ package Sidef::Object::Object {
             ($obj1, $obj2) = ($obj2, $obj1);
         }
 
-        if (   CORE::ref($obj1) && $obj1->SUPER::isa(CORE::ref($obj2))
-            or CORE::ref($obj2) && $obj2->SUPER::isa(CORE::ref($obj1))) {
-            if (defined(my $sub = $obj1->can('<=>'))) {
+        if (   CORE::ref($obj1) && UNIVERSAL::isa($obj1, CORE::ref($obj2))
+            or CORE::ref($obj2) && UNIVERSAL::isa($obj2, CORE::ref($obj1))) {
+            if (defined(my $sub = UNIVERSAL::can($obj1, '<=>'))) {
                 return $sub->($obj1, $obj2);
             }
         }
@@ -56,20 +56,69 @@ package Sidef::Object::Object {
       q{eq} => sub {
         my ($obj1, $obj2) = @_;
 
-        (   $obj1->SUPER::isa(CORE::ref($obj2) || return (Sidef::Types::Bool::Bool::FALSE))
-         || $obj2->SUPER::isa(CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
+        (   UNIVERSAL::isa($obj1, CORE::ref($obj2) || return (Sidef::Types::Bool::Bool::FALSE))
+         || UNIVERSAL::isa($obj2, CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
           || return (Sidef::Types::Bool::Bool::FALSE);
 
         return Sidef::Types::Bool::Bool::TRUE
           if (Scalar::Util::refaddr($obj1) == Scalar::Util::refaddr($obj2));
 
-        if (defined(my $sub = $obj1->can('=='))) {
+        if (defined(my $sub = UNIVERSAL::can($obj1, '=='))) {
             return $sub->($obj1, $obj2);
         }
 
         ($obj1 cmp $obj2)->is_zero;
       };
 
+    # Support for cyclic references
+    # (quite slow -- we don't use it for now)
+#<<<
+    #~ sub __eq__ {
+        #~ my %addr;
+
+        #~ my $sub = sub {
+            #~ my ($obj1, $obj2) = @_;
+
+            #~ my $refaddr1 = Scalar::Util::refaddr($obj1);
+            #~ my $refaddr2 = Scalar::Util::refaddr($obj2);
+
+            #~ if ($refaddr1 > $refaddr2) {
+                #~ ($refaddr1, $refaddr2) = ($refaddr2, $refaddr1);
+            #~ }
+
+            #~ (   $obj1->SUPER::isa(CORE::ref($obj2) || return (Sidef::Types::Bool::Bool::FALSE))
+             #~ || $obj2->SUPER::isa(CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
+              #~ || return (Sidef::Types::Bool::Bool::FALSE);
+
+            #~ return Sidef::Types::Bool::Bool::TRUE
+              #~ if ($refaddr1 == $refaddr2);
+
+            #~ if (defined(my $sub = $obj1->can('=='))) {
+
+                #~ my $key = join(' ', $refaddr1, $refaddr2);
+
+                #~ exists($addr{$key})
+                  #~ && return $addr{$key};
+
+                #~ my $bool = 1;
+                #~ my $bool_obj = bless \$bool, 'Sidef::Types::Bool::Bool';
+
+                #~ $addr{$key} = $bool_obj;
+                #~ $bool = $sub->($obj1, $obj2);
+                #~ return $bool_obj;
+            #~ }
+
+            #~ ($obj1 cmp $obj2)->is_zero;
+        #~ };
+
+        #~ eval 'use overload "eq" => $sub;';
+        #~ my $result = $sub->(@_);
+        #~ eval 'use overload "eq" => \&__eq__;';
+
+        #~ $result;
+    #~ }
+#>>>
+
     sub new {
         bless {}, __PACKAGE__;
     }
@@ -95,7 +144,7 @@ package Sidef::Object::Object {
 
     sub method {
         my ($self, $method, @args) = @_;
-        Sidef::Object::LazyMethod->new(obj => $self, method => $method, args => \@args);
+        Sidef::Object::LazyMethod->new({obj => $self, method => "$method", args => \@args});
     }
 
     sub object_id {
@@ -103,6 +152,8 @@ package Sidef::Object::Object {
         Sidef::Types::Number::Number->new(Scalar::Util::refaddr($self));
     }
 
+    *refaddr = \&object_id;
+
     sub class {
         my ($obj) = @_;
         my $ref = CORE::ref($obj) || $obj;
@@ -188,14 +239,14 @@ package Sidef::Object::Object {
 
     sub respond_to {
         my ($self, $method) = @_;
-        ($self->can($method))
+        UNIVERSAL::can($self, "$method")
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub is_a {
         my ($self, $obj) = @_;
-        ($self->SUPER::isa(CORE::ref($obj) || $obj))
+        UNIVERSAL::isa($self, CORE::ref($obj) || $obj)
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
     }
