@@ -23,19 +23,33 @@ package Sidef::Types::Array::Array {
     *call = \&new;
 
     sub get_value {
-        my ($self) = @_;
+        my %addr;
 
-        my @array;
-        foreach my $item (@{$self}) {
-            if (index(ref($item), 'Sidef::') == 0) {
-                CORE::push(@array, $item->get_value);
-            }
-            else {
-                CORE::push(@array, $item);
-            }
-        }
+        my $sub = sub {
+            my ($obj) = @_;
 
-        \@array;
+            my $refaddr = Scalar::Util::refaddr($obj);
+
+            exists($addr{$refaddr})
+              && return $addr{$refaddr};
+
+            my @array;
+            $addr{$refaddr} = \@array;
+
+            foreach my $item (@$obj) {
+                if (index(ref($item), 'Sidef::') == 0) {
+                    CORE::push(@array, $item->get_value);
+                }
+                else {
+                    CORE::push(@array, $item);
+                }
+            }
+
+            $addr{$refaddr};
+        };
+
+        local *Sidef::Types::Array::Array::get_value = $sub;
+        $sub->($_[0]);
     }
 
     sub unroll_operator {
@@ -44,18 +58,17 @@ package Sidef::Types::Array::Array {
         $operator = "$operator" if ref($operator);
 
         my @array;
-        if (defined $arg) {
-            my $argc  = @{$arg};
-            my $selfc = @{$self};
-            my $max   = $argc > $selfc ? $argc - 1 : $selfc - 1;
-            foreach my $i (0 .. $max) {
-                CORE::push(@array, $self->[$i % $selfc]->$operator($arg->[$i % $argc]));
-            }
-        }
-        else {
-            foreach my $i (0 .. $#{$self}) {
-                CORE::push(@array, $self->[$i]->$operator);
-            }
+
+        my @arg  = @$arg;
+        my @self = @$self;
+
+        (my $argc = @arg) || return $self->new(\@self);
+        my $selfc = @self;
+
+        my $max = $argc > $selfc ? $argc - 1 : $selfc - 1;
+
+        foreach my $i (0 .. $max) {
+            CORE::push(@array, $self[$i % $selfc]->$operator($arg[$i % $argc]));
         }
 
         $self->new(\@array);
@@ -105,17 +118,19 @@ package Sidef::Types::Array::Array {
 
         $operator = "$operator" if ref($operator);
 
+        my @arg = @{$arg};
+
         my @array;
         if ($operator eq '') {
             foreach my $i (@{$self}) {
-                foreach my $j (@{$arg}) {
+                foreach my $j (@arg) {
                     CORE::push(@array, $self->new($i, $j));
                 }
             }
         }
         else {
             foreach my $i (@{$self}) {
-                foreach my $j (@{$arg}) {
+                foreach my $j (@arg) {
                     CORE::push(@array, $i->$operator($j));
                 }
             }
@@ -129,19 +144,22 @@ package Sidef::Types::Array::Array {
 
         $operator = "$operator" if ref($operator);
 
-        my $self_len = $#{$self};
-        my $arg_len  = $#{$arg};
+        my @arg  = @$arg;
+        my @self = @$self;
+
+        my $self_len = $#self;
+        my $arg_len  = $#arg;
         my $min      = $self_len < $arg_len ? $self_len : $arg_len;
 
         my @array;
         if ($operator eq '') {
             foreach my $i (0 .. $min) {
-                CORE::push(@array, $self->new([$self->[$i], $arg->[$i]]));
+                CORE::push(@array, $self->new([$self[$i], $arg[$i]]));
             }
         }
         else {
             foreach my $i (0 .. $min) {
-                CORE::push(@array, $self->[$i]->$operator($arg->[$i]));
+                CORE::push(@array, $self[$i]->$operator($arg[$i]));
             }
         }
 
@@ -150,12 +168,6 @@ package Sidef::Types::Array::Array {
 
     sub mul {
         my ($self, $num) = @_;
-
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $num = $num->get_value;
-        }
-
         $self->new([(@{$self}) x $num]);
     }
 
@@ -165,10 +177,7 @@ package Sidef::Types::Array::Array {
         my @obj = @{$self};
 
         my @array;
-        my $len = @obj / do {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $num->get_value;
-        };
+        my $len = @obj / CORE::int($num);
 
         my $i   = 1;
         my $pos = $len;
@@ -406,10 +415,7 @@ package Sidef::Types::Array::Array {
     sub combinations {
         my ($self, $k, $block) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $k = $k->get_value;
-        }
+        $k = CORE::int($k);
 
         if (defined($block)) {
 
@@ -460,8 +466,6 @@ package Sidef::Types::Array::Array {
         $self->new(\@result);
     }
 
-    *combination = \&combinations;
-
     sub count {
         my ($self, $obj) = @_;
 
@@ -539,14 +543,7 @@ package Sidef::Types::Array::Array {
 
     sub make {
         my ($self, $size, $type) = @_;
-        $self->new(
-            [
-             ($type) x do {
-                 local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                 $size->get_value;
-               }
-            ]
-        );
+        $self->new([($type) x $size]);
     }
 
     sub _min_max {
@@ -609,14 +606,13 @@ package Sidef::Types::Array::Array {
     sub _min_max_by {
         my ($self, $code, $value) = @_;
 
-        @{$self} || return;
+        @$self || return;
 
-        my @pairs = map { [$_, scalar $code->run($_)] } @{$self};
+        my @pairs = map { [$_, scalar $code->run($_)] } @$self;
         my $item = $pairs[0];
 
         foreach my $i (1 .. $#pairs) {
-            my $val = $pairs[$i][1];
-            $item = $pairs[$i] if (($val cmp $item->[1]) eq $value);
+            $item = $pairs[$i] if (($pairs[$i][1] cmp $item->[1]) eq $value);
         }
 
         $item->[0];
@@ -652,11 +648,7 @@ package Sidef::Types::Array::Array {
             }
 
             my $max = $#{$self};
-            $arg = do {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $arg->get_value;
-              }
-              - 1;
+            $arg = CORE::int($arg) - 1;
             return $self->new(@{$self}[0 .. ($arg > $max ? $max : $arg)]);
         }
 
@@ -672,10 +664,7 @@ package Sidef::Types::Array::Array {
                 return $self->last_by($arg);
             }
 
-            my $from = @{$self} - do {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $arg->get_value;
-            };
+            my $from = @{$self} - CORE::int($arg);
             return $self->new(@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]);
         }
 
@@ -710,30 +699,18 @@ package Sidef::Types::Array::Array {
 
     sub exists {
         my ($self, $index) = @_;
-        (
-         exists $self->[
-           do {
-               local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-               $index->get_value;
-           }
-         ]
-        ) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        exists($self->[$index])
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     *has_index = \&exists;
 
     sub defined {
         my ($self, $index) = @_;
-        (
-         defined(
-             $self->[
-               do {
-                   local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                   $index->get_value;
-               }
-             ]
-         )
-        ) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        defined($self->[$index])
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub items {
@@ -768,11 +745,8 @@ package Sidef::Types::Array::Array {
 
         my $max = @{$self};
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $from = defined($from) ? ($from->get_value) : 0;
-            $to   = defined($to)   ? ($to->get_value)   : $max - 1;
-        }
+        $from = defined($from) ? CORE::int($from) : 0;
+        $to   = defined($to)   ? CORE::int($to)   : $max - 1;
 
         if (abs($from) > $max) {
             return;
@@ -816,10 +790,7 @@ package Sidef::Types::Array::Array {
     sub each_slice {
         my ($self, $n, $code) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $n = $n->get_value;
-        }
+        $n = CORE::int($n);
 
         my $end = @{$self};
         for (my $i = $n - 1 ; $i < $end ; $i += $n) {
@@ -867,10 +838,7 @@ package Sidef::Types::Array::Array {
     sub slices {
         my ($self, $n) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $n = $n->get_value;
-        }
+        $n = CORE::int($n);
 
         my @slices;
         my $end = @{$self};
@@ -889,10 +857,7 @@ package Sidef::Types::Array::Array {
     sub each_cons {
         my ($self, $n, $code) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $n = $n->get_value;
-        }
+        $n = CORE::int($n);
 
         foreach my $i ($n - 1 .. $#{$self}) {
             $code->run($self->new([@{$self}[$i - $n + 1 .. $i]]));
@@ -1116,7 +1081,7 @@ package Sidef::Types::Array::Array {
                     $obj->run($self->[$i])
                       && return Sidef::Types::Number::Number::_new_uint($i);
                 }
-                return Sidef::Types::Number::Number::_new_int(-1);
+                return Sidef::Types::Number::Number::MONE;
             }
 
             foreach my $i (0 .. $#{$self}) {
@@ -1124,10 +1089,12 @@ package Sidef::Types::Array::Array {
                   and return Sidef::Types::Number::Number::_new_uint($i);
             }
 
-            return Sidef::Types::Number::Number::_new_int(-1);
+            return Sidef::Types::Number::Number::MONE;
         }
 
-        Sidef::Types::Number::Number::_new_int(@{$self} ? 0 : -1);
+        @$self
+          ? Sidef::Types::Number::Number::ZERO
+          : Sidef::Types::Number::Number::MONE;
     }
 
     *index_by    = \&index;
@@ -1143,7 +1110,7 @@ package Sidef::Types::Array::Array {
                       && return Sidef::Types::Number::Number::_new_uint($i);
                 }
 
-                return Sidef::Types::Number::Number::_new_int(-1);
+                return Sidef::Types::Number::Number::MONE;
             }
 
             for (my $i = $#{$self} ; $i >= 0 ; $i--) {
@@ -1151,10 +1118,10 @@ package Sidef::Types::Array::Array {
                   and return Sidef::Types::Number::Number::_new_uint($i);
             }
 
-            return Sidef::Types::Number::Number::_new_int(-1);
+            return Sidef::Types::Number::Number::MONE;
         }
 
-        Sidef::Types::Number::Number::_new_int($#{$self});
+        $self->end;
     }
 
     *rindex_by  = \&rindex;
@@ -1233,19 +1200,20 @@ package Sidef::Types::Array::Array {
     *size = \&length;
 
     sub end {
-        my ($self) = @_;
-        Sidef::Types::Number::Number::_new_int($#{$self});
+        my $end = $#{$_[0]};
+            $end == -1 ? Sidef::Types::Number::Number::MONE
+          : $end == 0  ? Sidef::Types::Number::Number::ZERO
+          : $end == 1  ? Sidef::Types::Number::Number::ONE
+          : $end > 0   ? Sidef::Types::Number::Number::_new_uint($end)
+          :              Sidef::Types::Number::Number::_new_int($end);
     }
 
     *offset = \&end;
 
     sub resize {
         my ($self, $num) = @_;
-        $#{$self} = do {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $num->get_value;
-        };
-        $num;
+        $#{$self} = $num;
+        $self;
     }
 
     *resize_to = \&resize;
@@ -1254,11 +1222,10 @@ package Sidef::Types::Array::Array {
         my ($self, $amount) = @_;
 
         if (defined $amount) {
-            {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $amount = $amount->get_value;
-            }
-            return $self->new([map { $self->[CORE::rand(scalar @{$self})] } 1 .. $amount]);
+            $amount = CORE::int($amount);
+            return @{$self}
+              ? $self->new([map { $self->[CORE::rand(scalar @{$self})] } 1 .. $amount])
+              : Sidef::Types::Array::Array->new([]);
         }
         $self->[CORE::rand(scalar @{$self})];
     }
@@ -1268,18 +1235,10 @@ package Sidef::Types::Array::Array {
 
     sub range {
         my ($self) = @_;
-        Sidef::Types::Range::RangeNumber->__new__(
-            from => ${(Sidef::Types::Number::Number::ZERO)},
-            to   => do {
-                my $r = Math::GMPq::Rmpq_init();
-                my $i = $#{$self};
-                $i < 0
-                  ? Math::GMPq::Rmpq_set_si($r, $i, 1)
-                  : Math::GMPq::Rmpq_set_ui($r, $i, 1);
-                $r;
-            },
-            step => ${(Sidef::Types::Number::Number::ONE)},
-                                                 );
+        Sidef::Types::Range::RangeNumber->new(Sidef::Types::Number::Number::ZERO,
+                                              Sidef::Types::Number::Number::_new_int($#{$self}),
+                                              Sidef::Types::Number::Number::ONE,
+                                             );
     }
 
     sub indices {
@@ -1296,12 +1255,9 @@ package Sidef::Types::Array::Array {
     }
 
     sub insert {
-        my ($self, $index, @objects) = @_;
+        my ($self, $i, @objects) = @_;
 
-        my $i = do {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $index->get_value;
-        };
+        $i = CORE::int($i);
 
         if ($#{$self} < $i) {
             $#{$self} = $i - 1;
@@ -1534,18 +1490,7 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         if (defined $num) {
-            return $self->new(
-                [
-                 CORE::splice(
-                     @{$self},
-                     0,
-                     do {
-                         local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                         $num->get_value;
-                       }
-                 )
-                ]
-            );
+            return $self->new([CORE::splice(@{$self}, 0, $num,)]);
         }
 
         @{$self} || return;
@@ -1559,11 +1504,7 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         if (defined $num) {
-            {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $num = $num->get_value;
-            }
-
+            $num = CORE::int($num);
             $num = $num > $#{$self} ? 0 : @{$self} - $num;
             return $self->new([CORE::splice(@{$self}, $num)]);
         }
@@ -1583,14 +1524,7 @@ package Sidef::Types::Array::Array {
 
     sub delete_index {
         my ($self, $offset) = @_;
-        CORE::splice(
-            @{$self},
-            do {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $offset->get_value;
-            },
-            1
-                    );
+        CORE::splice(@{$self}, $offset, 1);
     }
 
     *pop_at    = \&delete_index;
@@ -1599,11 +1533,8 @@ package Sidef::Types::Array::Array {
     sub splice {
         my ($self, $offset, $length, @objects) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $offset = defined($offset) ? $offset->get_value : 0;
-            $length = defined($length) ? $length->get_value : scalar(@{$self});
-        }
+        $offset = defined($offset) ? CORE::int($offset) : 0;
+        $length = defined($length) ? CORE::int($length) : scalar(@$self);
 
         $self->new([CORE::splice(@{$self}, $offset, $length, @objects)]);
     }
@@ -1612,11 +1543,9 @@ package Sidef::Types::Array::Array {
         my ($self, $amount) = @_;
 
         my $end = $#{$self};
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $amount = $amount->get_value;
-            $amount = $end > ($amount - 1) ? $amount - 1 : $end;
-        }
+        $amount = CORE::int($amount);
+        $amount = $end > ($amount - 1) ? $amount - 1 : $end;
+
         $self->new([@{$self}[$end - $amount .. $end]]);
     }
 
@@ -1624,11 +1553,9 @@ package Sidef::Types::Array::Array {
         my ($self, $amount) = @_;
 
         my $end = $#{$self};
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $amount = $amount->get_value;
-            $amount = $end > ($amount - 1) ? $amount - 1 : $end;
-        }
+        $amount = CORE::int($amount);
+        $amount = $end > ($amount - 1) ? $amount - 1 : $end;
+
         $self->new([@{$self}[0 .. $amount]]);
     }
 
@@ -1756,11 +1683,7 @@ package Sidef::Types::Array::Array {
     sub rotate {
         my ($self, $num) = @_;
 
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $num = $num->get_value;
-        }
-
+        $num = CORE::int($num);
         $num %= ($#{$self} + 1);
         return $self->new([@{$self}]) if $num == 0;
 
@@ -1803,7 +1726,7 @@ package Sidef::Types::Array::Array {
         $self->new([CORE::reverse @{$self}]);
     }
 
-    *reversed = \&reverse;    # alias
+    *flip = \&reverse;
 
     sub to_hash {
         my ($self) = @_;
@@ -1812,6 +1735,12 @@ package Sidef::Types::Array::Array {
 
     *to_h = \&to_hash;
 
+    sub to_a {
+        $_[0];
+    }
+
+    *to_array = \&to_a;
+
     sub copy {
         my ($self) = @_;
         state $x = warn "[WARN] Array.copy() is deprecated: use .clone() or .dclone() instead!\n";
