@@ -11,8 +11,7 @@ package Sidef::Types::Block::Block {
     use overload
       q{bool} => sub { 1 },
       q{&{}}  => sub { $_[0]->{code} },
-      q{0+}   => sub { state $x = require Scalar::Util; Scalar::Util::refaddr($_[0]) },
-      q{""} => sub {
+      q{""}   => sub {
         my ($self) = @_;
 
         state $x = require Scalar::Util;
@@ -52,6 +51,10 @@ package Sidef::Types::Block::Block {
 
       OUTER: foreach my $method ($self, (exists($self->{kids}) ? @{$self->{kids}} : ())) {
 
+            if ($method->{type} eq 'block') {
+                return ($method, $method->{code}(@args));
+            }
+
             my $table = $self->{table};
 
             my %seen;
@@ -292,8 +295,11 @@ package Sidef::Types::Block::Block {
     sub while {
         my ($self, $condition) = @_;
 
-        while ($condition->{code}->()) {
-            $self->{code}->();
+        my $block = $self->{code};
+        $condition = $condition->{code};
+
+        while ($condition->()) {
+            $block->();
         }
 
         $self;
@@ -302,8 +308,10 @@ package Sidef::Types::Block::Block {
     sub loop {
         my ($self) = @_;
 
+        my $code = $self->{code};
+
         while (1) {
-            $self->{code}->();
+            $code->();
         }
 
         $self;
@@ -311,12 +319,7 @@ package Sidef::Types::Block::Block {
 
     sub if {
         my ($self, $bool) = @_;
-
-        if ($bool) {
-            return $self->{code}->();
-        }
-
-        $bool;
+        $bool ? $self->{code}->() : $bool;
     }
 
     sub __fdump {
@@ -410,8 +413,9 @@ package Sidef::Types::Block::Block {
             $args[0]->each($self);
         }
         else {
+            my $code = $self->{code};
             foreach my $item (@args) {
-                $self->{code}->($item);
+                $code->($item);
             }
             $self;
         }
