@@ -22,14 +22,15 @@ package Sidef::Types::Glob::Socket {
             *{__PACKAGE__ . '::' . $name} = sub {
                 my ($self, @args) = @_;
 
-                my $func    = \&{'Socket' . '::' . $name};
-                my @results = eval {
-                    $func->(map { $_->get_value } @args);
-                };
+                my $func = \&{'Socket' . '::' . $name};
+                @args = map { $_->get_value } @args;
+
+                my @results = eval { $func->(@args) };
+
                 if ($@) {
-                    my $result = $func->(map { $_->get_value } @args);
-                    @results = $result;
+                    @results = scalar $func->(@args);
                 }
+
                 return (
                         @results > 1
                         ? (map { Sidef::Perl::Perl->to_sidef($_) } @results)
@@ -41,24 +42,12 @@ package Sidef::Types::Glob::Socket {
 
     sub open {
         my ($self, $domain, $type, $protocol) = @_;
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $domain   = $domain->get_value;
-            $type     = $type->get_value;
-            $protocol = $protocol->get_value;
-        }
         CORE::socket(my $fh, $domain, $type, $protocol) || return;
         Sidef::Types::Glob::SocketHandle->new(fh => $fh);
     }
 
     sub socketpair {
         my ($self, $socket1, $socket2, $domain, $type, $protocol) = @_;
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $domain   = $domain->get_value;
-            $type     = $type->get_value;
-            $protocol = $protocol->get_value;
-        }
         CORE::socketpair(my $sh1, my $sh2, $domain, $type, $protocol)
           || return (Sidef::Types::Bool::Bool::FALSE);
         ${$socket1} = Sidef::Types::Glob::SocketHandle->new(fh => $sh1);
@@ -71,12 +60,12 @@ package Sidef::Types::Glob::Socket {
     #
     sub gethostbyname {
         my ($self, $name) = @_;
-        Sidef::Types::String::String->new(CORE::gethostbyname($name->get_value) // return);
+        Sidef::Types::String::String->new(CORE::gethostbyname("$name") // return);
     }
 
     sub gethostbyaddr {
         my ($self, $addr, $addrtype) = @_;
-        Sidef::Types::String::String->new(CORE::gethostbyaddr($addr->get_value, $addrtype->get_value) // return);
+        Sidef::Types::String::String->new(CORE::gethostbyaddr("$addr", "$addrtype") // return);
     }
 
     sub gethostent {
@@ -89,12 +78,12 @@ package Sidef::Types::Glob::Socket {
     #
     sub getnetbyname {
         my ($self, $name) = @_;
-        Sidef::Types::String::String->new(CORE::getnetbyname($name->get_value) // return);
+        Sidef::Types::String::String->new(CORE::getnetbyname("$name") // return);
     }
 
     sub getnetbyaddr {
         my ($self, $addr, $addrtype) = @_;
-        Sidef::Types::String::String->new(CORE::getnetbyaddr($addr->get_value, $addrtype->get_value) // return);
+        Sidef::Types::String::String->new(CORE::getnetbyaddr("$addr", "$addrtype") // return);
     }
 
     sub getnetent {
@@ -107,24 +96,11 @@ package Sidef::Types::Glob::Socket {
     #
     sub getservbyname {
         my ($self, $name, $proto) = @_;
-        Sidef::Types::String::String->new(
-            CORE::getservbyname(
-                $name->get_value,
-                do {
-                    local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                    $proto->get_value;
-                  }
-              ) // return
-        );
+        Sidef::Types::String::String->new(CORE::getservbyname("$name", $proto) // return);
     }
 
     sub getservbyport {
         my ($self, $port, $proto) = @_;
-        {
-            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-            $port  = $port->get_value;
-            $proto = $proto->get_value;
-        }
         Sidef::Types::String::String->new(CORE::getservbyport($port, $proto) // return);
     }
 
@@ -138,14 +114,7 @@ package Sidef::Types::Glob::Socket {
     #
     sub getprotobynumber {
         my ($self, $num) = @_;
-        Sidef::Types::String::String->new(
-            CORE::getprotobynumber(
-                do {
-                    local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                    $num->get_value;
-                  }
-              ) // return
-        );
+        Sidef::Types::String::String->new(CORE::getprotobynumber(CORE::int($num)) // return);
     }
 
     sub getprotobyname {
@@ -163,22 +132,22 @@ package Sidef::Types::Glob::Socket {
     #
     sub sethostent {
         my ($self, $stayopen) = @_;
-        (CORE::sethostent($stayopen->get_value)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        (CORE::sethostent($stayopen ? 1 : 0)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub setnetent {
         my ($self, $stayopen) = @_;
-        (CORE::setnetent($stayopen->get_value)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        (CORE::setnetent($stayopen ? 1 : 0)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub setprotoent {
         my ($self, $stayopen) = @_;
-        (CORE::setprotoent($stayopen->get_value)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        (CORE::setprotoent($stayopen ? 1 : 0)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub setservent {
         my ($self, $stayopen) = @_;
-        (CORE::setservent($stayopen->get_value)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        (CORE::setservent($stayopen ? 1 : 0)) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 };
 
