@@ -9,6 +9,7 @@ package Sidef::Types::Hash::Hash {
 
     use overload
       q{bool} => sub { scalar(CORE::keys %{$_[0]}) },
+      q{0+}   => sub { scalar(CORE::keys %{$_[0]}) },
       q{""}   => \&dump;
 
     use Sidef::Types::Bool::Bool;
@@ -21,19 +22,33 @@ package Sidef::Types::Hash::Hash {
     *call = \&new;
 
     sub get_value {
-        my ($self) = @_;
+        my %addr;
 
-        my %hash;
-        foreach my $k (CORE::keys %$self) {
-            my $v = $self->{$k};
-            $hash{$k} = (
-                         index(ref($v), 'Sidef::') == 0
-                         ? $v->get_value
-                         : $v
-                        );
-        }
+        my $sub = sub {
+            my ($obj) = @_;
 
-        \%hash;
+            my $refaddr = Scalar::Util::refaddr($obj);
+
+            exists($addr{$refaddr})
+              && return $addr{$refaddr};
+
+            my %hash;
+            $addr{$refaddr} = \%hash;
+
+            foreach my $k (CORE::keys %$obj) {
+                my $v = $obj->{$k};
+                $hash{$k} = (
+                             index(ref($v), 'Sidef::') == 0
+                             ? $v->get_value
+                             : $v
+                            );
+            }
+
+            $addr{$refaddr};
+        };
+
+        local *Sidef::Types::Hash::Hash::get_value = $sub;
+        $sub->($_[0]);
     }
 
     sub items {
@@ -327,7 +342,7 @@ package Sidef::Types::Hash::Hash {
         }
 
         Sidef::Types::Array::Array->new(
-                   [map { Sidef::Types::Array::Array->new($_->[1], $self->{$_->[0]}) } (sort { $a->[2] cmp $b->[2] } @array)]);
+                    [map { Sidef::Types::Array::Pair->new($_->[1], $self->{$_->[0]}) } (sort { $a->[2] cmp $b->[2] } @array)]);
     }
 
     sub sort {
@@ -337,7 +352,7 @@ package Sidef::Types::Hash::Hash {
             return
               Sidef::Types::Array::Array->new(
                                               [
-                                               map { Sidef::Types::Array::Array->new($_->[1], $self->{$_->[0]}) } (
+                                               map { Sidef::Types::Array::Pair->new($_->[1], $self->{$_->[0]}) } (
                                                         sort { scalar $code->run($a->[1], $b->[1]) }
                                                           map { [$_, Sidef::Types::String::String->new($_)] } CORE::keys %$self
                                                )
@@ -347,12 +362,35 @@ package Sidef::Types::Hash::Hash {
 
         Sidef::Types::Array::Array->new(
             map {
-                Sidef::Types::Array::Array->new(Sidef::Types::String::String->new($_), $self->{$_})
+                Sidef::Types::Array::Pair->new(Sidef::Types::String::String->new($_), $self->{$_})
               }
               sort CORE::keys %$self
         );
     }
 
+    sub _min_max {
+        my ($self, $code, $value) = @_;
+
+        my @pairs = map { [$_, $code->run(Sidef::Types::String::String->new($_), $self->{$_})] } keys %$self;
+
+        my $item = $pairs[0];
+        foreach my $i (1 .. $#pairs) {
+            $item = $pairs[$i] if (($pairs[$i][1] cmp $item->[1]) eq $value);
+        }
+
+        Sidef::Types::Array::Pair->new(Sidef::Types::String::String->new($item->[0]), $self->{$item->[0]});
+    }
+
+    sub max_by {
+        my ($self, $code) = @_;
+        $self->_min_max($code, Sidef::Types::Number::Number::ONE);
+    }
+
+    sub min_by {
+        my ($self, $code) = @_;
+        $self->_min_max($code, Sidef::Types::Number::Number::MONE);
+    }
+
     sub to_a {
         my ($self) = @_;
         Sidef::Types::Array::Array->new(
@@ -364,12 +402,23 @@ package Sidef::Types::Hash::Hash {
         );
     }
 
-    *pairs    = \&to_a;
     *to_array = \&to_a;
 
+    sub pairs {
+        my ($self, @keys) = @_;
+        Sidef::Types::Array::Array->new([map { Sidef::Types::Array::Pair->new($_, $self->{$_}) } @keys]);
+    }
+
+    sub pair {
+        my ($self, $key) = @_;
+        Sidef::Types::Array::Pair->new($key, $self->{$key});
+    }
+
     sub exists {
         my ($self, $key) = @_;
-        (CORE::exists $self->{$key}) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        CORE::exists($self->{$key})
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     *has_key  = \&exists;
@@ -378,7 +427,7 @@ package Sidef::Types::Hash::Hash {
     *include  = \&exists;
     *includes = \&exists;
 
-    sub flip {
+    sub reverse {
         my ($self) = @_;
 
         my $new_hash = $self->new();
@@ -388,7 +437,7 @@ package Sidef::Types::Hash::Hash {
         $new_hash;
     }
 
-    *reverse = \&flip;
+    *flip = \&reverse;
 
     sub copy {
         my ($self) = @_;
