@@ -39,8 +39,8 @@ package Sidef::Types::Number::Complex {
                 return $x->add(__PACKAGE__->new($y)->mul(i()));
             }
         }
-        elsif (index(ref($x), 'Sidef::') == 0) {
-            $x = "$x";
+        elsif (ref($x) eq 'SCALAR' or index(ref($x), 'Sidef::') == 0) {
+            $x = ref($x) eq 'SCALAR' ? $$x : "$x";
             if ($x eq 'i' or $x eq '+i') {
                 return __PACKAGE__->new(__PACKAGE__->new(0, 1), $y);
             }
@@ -81,8 +81,8 @@ package Sidef::Types::Number::Complex {
         elsif (ref($y) eq __PACKAGE__) {
             return $y->mul(i())->add(__PACKAGE__->new($x));
         }
-        elsif (index(ref($y), 'Sidef::') == 0) {
-            $y = "$y";
+        elsif (ref($y) eq 'SCALAR' or index(ref($y), 'Sidef::') == 0) {
+            $y = ref($y) eq 'SCALAR' ? $$y : "$y";
             if ($y eq 'i' or $y eq '+i') {
                 return __PACKAGE__->new($x, __PACKAGE__->new(0, 1));
             }
@@ -144,13 +144,24 @@ package Sidef::Types::Number::Complex {
 
     sub _valid {
         (
-         ref($_) eq __PACKAGE__
-           or die "[ERROR] Invalid argument `$_` of type "
-           . Sidef::normalize_type(ref($_)) . " in "
-           . Sidef::normalize_method((caller(1))[3])
-           . "(). Expected an argument of type Complex!\n"
-        )
-          for @_;
+         ref($$_) eq __PACKAGE__
+           or do {
+             ref($$_) eq 'Sidef::Types::Number::Number' ? do { $$_ = __PACKAGE__->new($$_) } : do {
+                 my $sub = UNIVERSAL::can($$_, 'to_c') // overload::Method($$_, '0+');
+
+                 my $tmp = (
+                            defined($sub)
+                            ? __PACKAGE__->new($sub->($$_))
+                            : die "[ERROR] Value <<$$_>> cannot be implicitly converted to a complex number!"
+                           );
+
+                 ref($tmp) eq __PACKAGE__
+                   or die "[ERROR] Cannot convert <<$$_>> to a complex number! (is method \"to_c\" well-defined?)";
+
+                 $$_ = $tmp;
+               }
+           }
+        ) for @_;
     }
 
     sub get_value {
@@ -292,11 +303,12 @@ package Sidef::Types::Number::Complex {
         my ($x, $y) = @_;
         my $r = Math::MPC::Rmpc_init2($PREC);
 
-        if (ref($y) eq __PACKAGE__) {
-            Math::MPC::Rmpc_add($r, $$x, $$y, $ROUND);
+        if (ref($y) eq 'Sidef::Types::Number::Number') {
+            Math::MPC::Rmpc_add_fr($r, $$x, $y->_big2mpfr(), $ROUND);
         }
         else {
-            Math::MPC::Rmpc_add_fr($r, $$x, $y->_big2mpfr(), $ROUND);
+            _valid(\$y);
+            Math::MPC::Rmpc_add($r, $$x, $$y, $ROUND);
         }
 
         bless(\$r, __PACKAGE__);
@@ -306,11 +318,12 @@ package Sidef::Types::Number::Complex {
         my ($x, $y) = @_;
         my $r = Math::MPC::Rmpc_init2($PREC);
 
-        if (ref($y) eq __PACKAGE__) {
-            Math::MPC::Rmpc_sub($r, $$x, $$y, $ROUND);
+        if (ref($y) eq 'Sidef::Types::Number::Number') {
+            Math::MPC::Rmpc_add_fr($r, $$x, -$y->_big2mpfr(), $ROUND);
         }
         else {
-            Math::MPC::Rmpc_add_fr($r, $$x, -$y->_big2mpfr(), $ROUND);
+            _valid(\$y);
+            Math::MPC::Rmpc_sub($r, $$x, $$y, $ROUND);
         }
 
         bless(\$r, __PACKAGE__);
@@ -320,11 +333,12 @@ package Sidef::Types::Number::Complex {
         my ($x, $y) = @_;
         my $r = Math::MPC::Rmpc_init2($PREC);
 
-        if (ref($y) eq __PACKAGE__) {
-            Math::MPC::Rmpc_mul($r, $$x, $$y, $ROUND);
+        if (ref($y) eq 'Sidef::Types::Number::Number') {
+            Math::MPC::Rmpc_mul_fr($r, $$x, $y->_big2mpfr(), $ROUND);
         }
         else {
-            Math::MPC::Rmpc_mul_fr($r, $$x, $y->_big2mpfr(), $ROUND);
+            _valid(\$y);
+            Math::MPC::Rmpc_mul($r, $$x, $$y, $ROUND);
         }
 
         bless(\$r, __PACKAGE__);
@@ -335,11 +349,12 @@ package Sidef::Types::Number::Complex {
 
         my $r = Math::MPC::Rmpc_init2($PREC);
 
-        if (ref($y) eq __PACKAGE__) {
-            Math::MPC::Rmpc_div($r, $$x, $$y, $ROUND);
+        if (ref($y) eq 'Sidef::Types::Number::Number') {
+            Math::MPC::Rmpc_div_fr($r, $$x, $y->_big2mpfr(), $ROUND);
         }
         else {
-            Math::MPC::Rmpc_div_fr($r, $$x, $y->_big2mpfr(), $ROUND);
+            _valid(\$y);
+            Math::MPC::Rmpc_div($r, $$x, $$y, $ROUND);
         }
 
         bless(\$r, __PACKAGE__);
@@ -358,11 +373,12 @@ package Sidef::Types::Number::Complex {
         my ($x, $y) = @_;
         my $r = Math::MPC::Rmpc_init2($PREC);
 
-        if (ref($y) eq __PACKAGE__) {
-            Math::MPC::Rmpc_pow($r, $$x, $$y, $ROUND);
+        if (ref($y) eq 'Sidef::Types::Number::Number') {
+            Math::MPC::Rmpc_pow_fr($r, $$x, $y->_big2mpfr(), $ROUND);
         }
         else {
-            Math::MPC::Rmpc_pow_fr($r, $$x, $y->_big2mpfr(), $ROUND);
+            _valid(\$y);
+            Math::MPC::Rmpc_pow($r, $$x, $$y, $ROUND);
         }
 
         bless(\$r, __PACKAGE__);
@@ -399,16 +415,17 @@ package Sidef::Types::Number::Complex {
         Math::MPC::Rmpc_log($r, $$x, $ROUND);
 
         if (defined $y) {
-            if (ref($y) eq __PACKAGE__) {
-                my $baseln = Math::MPC::Rmpc_init2($PREC);
-                Math::MPC::Rmpc_log($baseln, $$y, $ROUND);
-                Math::MPC::Rmpc_div($r, $r, $baseln, $ROUND);
-            }
-            else {
+            if (ref($y) eq 'Sidef::Types::Number::Number') {
                 my $baseln = $y->_big2mpfr();
                 Math::MPFR::Rmpfr_log($baseln, $baseln, $Sidef::Types::Number::Number::ROUND);
                 Math::MPC::Rmpc_div_fr($r, $r, $baseln, $ROUND);
             }
+            else {
+                _valid(\$y);
+                my $baseln = Math::MPC::Rmpc_init2($PREC);
+                Math::MPC::Rmpc_log($baseln, $$y, $ROUND);
+                Math::MPC::Rmpc_div($r, $r, $baseln, $ROUND);
+            }
         }
 
         bless(\$r, __PACKAGE__);
@@ -723,7 +740,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         my $r = Math::MPC::Rmpc_init2($PREC);
@@ -743,7 +760,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         if (Math::MPC::Rmpc_cmp($$x, $$y) == 0) {
@@ -761,7 +778,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         if (Math::MPC::Rmpc_cmp($$x, $$y) == 0) {
@@ -779,7 +796,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         $x->abs->gt($y->abs);
@@ -792,7 +809,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         $x->abs->ge($y->abs);
@@ -805,7 +822,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         $x->abs->lt($y->abs);
@@ -818,7 +835,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         $x->abs->le($y->abs);
@@ -831,7 +848,7 @@ package Sidef::Types::Number::Complex {
             $y = __PACKAGE__->new($y);
         }
         else {
-            _valid($y);
+            _valid(\$y);
         }
 
         $x->abs->cmp($y->abs);
