@@ -4,8 +4,6 @@ package Sidef::Types::Number::Inf {
     use 5.014;
 
     use parent qw(
-      Sidef::Object::Object
-      Sidef::Convert::Convert
       Sidef::Types::Number::Number
       );
 
@@ -14,26 +12,27 @@ package Sidef::Types::Number::Inf {
       q{0+}   => sub { 'inf' },
       q{""}   => sub { 'Inf' };
 
-    state $INF = do {
-        my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_set_ui($r, 1, 0);
-        bless \$r, __PACKAGE__;
-    };
+    my $INF = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_ui($INF, 1, 0);
 
-    sub new { $INF }
+    use constant INF => bless(\$INF, __PACKAGE__);
+
+    use Sidef::Types::Number::Nan;
+
+    sub new { INF }
 
     sub get_value { 'Inf' }
 
     sub add {
         my ($x, $y) = @_;
-        ref($y) eq 'Sidef::Types::Number::Ninf' ? nan() : $x;
+        ref($y) eq 'Sidef::Types::Number::Ninf' ? Sidef::Types::Number::Nan::NAN : $x;
     }
 
     *iadd = \&add;
 
     sub sub {
         my ($x, $y) = @_;
-        ref($y) eq __PACKAGE__ ? nan() : $x;
+        ref($y) eq __PACKAGE__ ? Sidef::Types::Number::Nan::NAN : $x;
     }
 
     *isub = \&sub;
@@ -41,7 +40,7 @@ package Sidef::Types::Number::Inf {
     sub mul {
         my ($x, $y) = @_;
         ref($y) eq __PACKAGE__ and return $x;
-        $y->is_neg ? $x->neg : $y->is_pos ? $x : nan();
+        $y->is_neg ? $x->neg : $y->is_pos ? $x : Sidef::Types::Number::Nan::NAN;
     }
 
     *imul = \&mul;
@@ -49,22 +48,18 @@ package Sidef::Types::Number::Inf {
     sub div {
         my ($x, $y) = @_;
         if (ref($y) eq __PACKAGE__ or ref($y) eq 'Sidef::Types::Number::Ninf') {
-            return nan();
+            return Sidef::Types::Number::Nan::NAN;
         }
         $y->is_neg ? $x->neg : $x;
     }
 
     *idiv = \&div;
 
-    sub is_pos {
-        (Sidef::Types::Bool::Bool::TRUE);
-    }
+    sub is_pos { Sidef::Types::Bool::Bool::TRUE }
 
     *is_inf = \&is_pos;
 
-    sub is_ninf {
-        (Sidef::Types::Bool::Bool::FALSE);
-    }
+    sub is_ninf { Sidef::Types::Bool::Bool::FALSE }
 
     *is_nan    = \&is_ninf;
     *is_neg    = \&is_ninf;
@@ -82,9 +77,7 @@ package Sidef::Types::Number::Inf {
     *is_one    = \&is_ninf;
     *is_mone   = \&is_ninf;
 
-    sub nan {
-        state $x = Sidef::Types::Number::Nan->new;
-    }
+    sub nan { Sidef::Types::Number::Nan::NAN }
 
     *mod         = \&nan;
     *imod        = \&nan;
@@ -121,7 +114,7 @@ package Sidef::Types::Number::Inf {
 
     sub divmod {
         my ($x, $y) = @_;
-        ($x->div($y), nan());
+        ($x->div($y), Sidef::Types::Number::Nan::NAN);
     }
 
     sub ninf {
@@ -135,13 +128,13 @@ package Sidef::Types::Number::Inf {
         my ($x, $y) = @_;
         ref($y) eq 'Sidef::Types::Number::Inf' || ref($y) eq 'Sidef::Types::Number::Ninf' ? Sidef::Types::Number::Number::ONE
           : $y->is_neg ? Sidef::Types::Number::Number::ZERO
-          :              $x->inf();
+          :              INF;
     }
 
     *iroot = \&root;
 
     sub min { $_[1] }
-    sub inf { $_[0] }
+    sub inf { INF }
 
     *max              = \&inf;
     *abs              = \&inf;
@@ -191,7 +184,6 @@ package Sidef::Types::Number::Inf {
     *lucas            = \&inf;
     *shift_left       = \&inf;
     *shift_right      = \&inf;
-    *rand_int         = \&inf;
     *irand            = \&inf;
     *rand             = \&inf;
     *rad2deg          = \&inf;
@@ -203,7 +195,7 @@ package Sidef::Types::Number::Inf {
     *round            = \&inf;
     *roundf           = \&inf;
 
-    sub zero { (Sidef::Types::Number::Number::ZERO) }
+    sub zero { Sidef::Types::Number::Number::ZERO }
 
     *inv = \&zero;
 
@@ -218,22 +210,12 @@ package Sidef::Types::Number::Inf {
     *acoth = \&zero;
     *erfc  = \&zero;
 
-    sub tanh { (Sidef::Types::Number::Number::ONE) }
+    sub tanh { Sidef::Types::Number::Number::ONE }
 
     *coth = \&tanh;
     *zeta = \&tanh;
     *erf  = \&tanh;
 
-    sub array_to { Sidef::Types::Array::Array->new }
-
-    *arr_to       = \&array_to;
-    *arr_downto   = \&array_to;
-    *array_downto = \&array_to;
-
-    # Probably, this ones should return each a RangeNumber object instead
-    *to     = \&array_to;
-    *downto = \&array_to;
-
     sub chr  { state $x = Sidef::Types::String::String->new('') }
     sub sign { state $x = Sidef::Types::String::String->new('+') }
 
@@ -371,8 +353,7 @@ package Sidef::Types::Number::Inf {
     #
 
     sub eq {
-        my ($x, $y) = @_;
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($_[1]) eq __PACKAGE__) {
             (Sidef::Types::Bool::Bool::TRUE);
         }
         else {
@@ -381,8 +362,7 @@ package Sidef::Types::Number::Inf {
     }
 
     sub ne {
-        my ($x, $y) = @_;
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($_[1]) eq __PACKAGE__) {
             (Sidef::Types::Bool::Bool::FALSE);
         }
         else {
@@ -391,8 +371,7 @@ package Sidef::Types::Number::Inf {
     }
 
     sub gt {
-        my ($x, $y) = @_;
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($_[1]) eq __PACKAGE__) {
             (Sidef::Types::Bool::Bool::FALSE);
         }
         else {
@@ -409,8 +388,7 @@ package Sidef::Types::Number::Inf {
     }
 
     sub le {
-        my ($x, $y) = @_;
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($_[1]) eq __PACKAGE__) {
             (Sidef::Types::Bool::Bool::TRUE);
         }
         else {
@@ -419,17 +397,17 @@ package Sidef::Types::Number::Inf {
     }
 
     sub cmp {
-        my ($x, $y) = @_;
-        ref($y) eq __PACKAGE__ ? (Sidef::Types::Number::Number::ZERO) : (Sidef::Types::Number::Number::ONE);
+        ref($_[1]) eq __PACKAGE__
+          ? Sidef::Types::Number::Number::ZERO
+          : Sidef::Types::Number::Number::ONE;
     }
 
     sub i {
-        my ($x) = @_;
-        Sidef::Types::Number::Complex->new(0, $x);
+        state $x = Sidef::Types::Number::Complex->new(0, INF);
     }
 
     sub dump {
-        Sidef::Types::String::String->new('Inf');
+        state $x = Sidef::Types::String::String->new('Inf');
     }
 
     *commify = \&dump;
@@ -469,9 +447,6 @@ package Sidef::Types::Number::Inf {
         *{__PACKAGE__ . '::' . '=='}  = \&eq;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '≠'} = \&ne;
-        *{__PACKAGE__ . '::' . '..'}  = \&array_to;
-        *{__PACKAGE__ . '::' . '..^'} = \&to;
-        *{__PACKAGE__ . '::' . '^..'} = \&downto;
         *{__PACKAGE__ . '::' . '>'}   = \&gt;
         *{__PACKAGE__ . '::' . '>='}  = \&ge;
         *{__PACKAGE__ . '::' . '≥'} = \&ge;
