@@ -13,21 +13,18 @@ package Sidef::Types::Number::Nan {
       q{0+}   => sub { 'NaN' },
       q{""}   => sub { 'NaN' };
 
-    state $NAN = do {
-        my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_set_si($r, "-0", "-0");
-        bless \$r, __PACKAGE__;
-    };
+    my $NAN = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_si($NAN, "-0", "-0");
 
-    sub new { $NAN }
+    use constant NAN => bless(\$NAN, __PACKAGE__);
+
+    sub new { NAN }
 
     sub get_value { 'NaN' }
 
-    sub is_nan {
-        (Sidef::Types::Bool::Bool::TRUE);
-    }
+    sub is_nan { Sidef::Types::Bool::Bool::TRUE }
 
-    sub nan { $_[0] }
+    sub nan { NAN }
 
     *abs         = \&nan;
     *add         = \&nan;
@@ -105,9 +102,7 @@ package Sidef::Types::Number::Nan {
     *max         = \&nan;
     *min         = \&nan;
     *int         = \&nan;
-    *as_int      = \&nan;
     *float       = \&nan;
-    *as_float    = \&nan;
     *sqr         = \&nan;
     *next_pow    = \&nan;
     *next_pow2   = \&nan;
@@ -121,9 +116,8 @@ package Sidef::Types::Number::Nan {
     *legendre    = \&nan;
     *jacobi      = \&nan;
     *kronecker   = \&nan;
-    *rand_int    = \&nan;
-    *rand        = \&nan;
     *rand        = \&nan;
+    *irand       = \&nan;
     *rad2deg     = \&nan;
     *rad2grad    = \&nan;
     *deg2grad    = \&nan;
@@ -132,6 +126,8 @@ package Sidef::Types::Number::Nan {
     *grad2deg    = \&nan;
     *round       = \&nan;
     *roundf      = \&nan;
+    *re          = \&nan;
+    *real        = \&nan;
 
     sub cmp { }
 
@@ -140,7 +136,9 @@ package Sidef::Types::Number::Nan {
 
     sub eq {
         my ($x, $y) = @_;
-        ref($y) eq ref($x) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
+        ref($y) eq ref($x)
+          ? Sidef::Types::Bool::Bool::TRUE
+          : Sidef::Types::Bool::Bool::FALSE;
     }
 
     sub is_inf {
@@ -171,47 +169,55 @@ package Sidef::Types::Number::Nan {
     sub complex {
         my ($x, $y) = @_;
         if (defined $y) {
-            Sidef::Types::Number::Complex->new($x, $y);
+            Sidef::Types::Number::Complex->new(NAN, $y);
         }
         else {
-            Sidef::Types::Number::Complex->new($x);
+            state $z = Sidef::Types::Number::Complex->new(NAN);
         }
     }
 
     *c = \&complex;
 
     sub i {
-        my ($x) = @_;
-        Sidef::Types::Number::Complex->new(0, $x);
+        state $x = Sidef::Types::Number::Complex->new(0, NAN);
     }
 
     sub dump {
-        Sidef::Types::String::String->new('NaN');
+        state $x = Sidef::Types::String::String->new('NaN');
     }
 
     *commify = \&dump;
 
     sub chr {
-        Sidef::Types::String::String->new('');
+        state $x = Sidef::Types::String::String->new('');
     }
 
-    sub numerator {
+    *as_bin   = \&chr;
+    *as_hex   = \&chr;
+    *as_oct   = \&chr;
+    *as_int   = \&chr;
+    *as_float = \&chr;
+
+    sub zero {
         Sidef::Types::Number::Number::ZERO;
     }
 
-    *nu = \&numerator;
+    *nu        = \&zero;
+    *numerator = \&zero;
 
-    sub denominator {
-        Sidef::Types::Number::Number::ZERO;
-    }
+    *de          = \&zero;
+    *denominator = \&zero;
 
-    *de = \&denominator;
+    *im        = \&zero;
+    *imag      = \&zero;
+    *imaginary = \&zero;
 
     sub of {
-        Sidef::Types::Array::Array->new();
+        Sidef::Types::Array::Array->new([]);
     }
 
-    *defs = \&of;
+    *defs   = \&of;
+    *digits = \&of;
 
     sub times { $_[1] }
 
@@ -223,6 +229,8 @@ package Sidef::Types::Number::Nan {
 
     *nude = \&parts;
 
+    sub digits { Sidef::Types::Array::Array->new([]) }
+
     {
         no strict 'refs';
         *{__PACKAGE__ . '::' . '/'}   = \&div;
