@@ -10,12 +10,19 @@ package Sidef::Types::Number::Number {
     our $ROUND = Math::MPFR::MPFR_RNDN();
     our $PREC  = 128;
 
-    our $GET_PERL_VALUE = 0;
+    my $ONE = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_ui($ONE, 1, 1);
+
+    my $ZERO = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_ui($ZERO, 0, 1);
+
+    my $MONE = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_si($MONE, -1, 1);
 
     use constant {
-                  ONE  => bless(\Math::GMPq->new(1),  __PACKAGE__),
-                  ZERO => bless(\Math::GMPq->new(0),  __PACKAGE__),
-                  MONE => bless(\Math::GMPq->new(-1), __PACKAGE__),
+                  ONE  => bless(\$ONE,  __PACKAGE__),
+                  ZERO => bless(\$ZERO, __PACKAGE__),
+                  MONE => bless(\$MONE, __PACKAGE__),
                  };
 
     use parent qw(
@@ -25,25 +32,33 @@ package Sidef::Types::Number::Number {
 
     use overload
       q{bool} => sub { Math::GMPq::Rmpq_sgn(${$_[0]}) != 0 },
-      q{0+}   => sub { Math::GMPq::Rmpq_get_d(${$_[0]}) },
-      q{""}   => \&get_value;
+      q{0+}   => \&get_value,
+      q{""}   => \&_as_str;
 
     use Sidef::Types::Bool::Bool;
 
+    my @cache;
+
     sub _new {
         bless(\$_[0], __PACKAGE__);
     }
 
     sub _new_int {
+        $_[0] == -1 && return MONE;
+        $_[0] >= 0  && return &_new_uint;
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_set_si($r, $_[0], 1);
-        bless \$r, __PACKAGE__;
+        bless(\$r, __PACKAGE__);
     }
 
     sub _new_uint {
+        exists($cache[$_[0]])
+          && return $cache[$_[0]];
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_set_ui($r, $_[0], 1);
-        bless \$r, __PACKAGE__;
+        $_[0] <= 8192
+          ? ($cache[$_[0]] = bless(\$r, __PACKAGE__))
+          : bless(\$r, __PACKAGE__);
     }
 
     sub _set_str {
@@ -61,14 +76,17 @@ package Sidef::Types::Number::Number {
           : ref($num) eq __PACKAGE__ ? $num
           : do {
 
-            $base = defined($base) ? ref($base) ? $base->get_value : $base : 10;
+            $base = defined($base) ? ref($base) ? "$base" : $base : 10;
 
-            $num = $num->get_value
+            $num = "$num"
               if (index(ref($num), 'Sidef::') == 0);
 
             my $r = Math::GMPq::Rmpq_init();
             my $rat = $num ? ($base == 10 && $num =~ tr/Ee.//) ? _str2rat($num) : ($num =~ tr/+//dr) : 0;
-            Math::GMPq::Rmpq_set_str($r, $rat, $base);
+            eval { Math::GMPq::Rmpq_set_str($r, $rat, $base) };
+            $@ && return nan();
+
+            #$@ && die "[ERROR] Value <<$num>> is not a valid base-$base number";
             Math::GMPq::Rmpq_canonicalize($r) if index($rat, '/') != -1;
             bless \$r, __PACKAGE__;
           };
@@ -78,13 +96,22 @@ package Sidef::Types::Number::Number {
 
     sub _valid {
         (
-         ref($_) eq __PACKAGE__
-           or die "[ERROR] Invalid argument `$_` of type "
-           . Sidef::normalize_type(ref($_)) . " in "
-           . Sidef::normalize_method((caller(1))[3])
-           . "(). Expected an argument of type Number!\n"
-        )
-          for @_;
+         ref($$_) eq __PACKAGE__
+           or do {
+             my $sub = overload::Method($$_, '0+');
+
+             my $tmp = (
+                        defined($sub)
+                        ? __PACKAGE__->new($sub->($$_))
+                        : die "[ERROR] Value <<$$_>> cannot be implicitly converted to a number!"
+                       );
+
+             ref($tmp) eq __PACKAGE__
+               or die "[ERROR] Cannot convert <<$$_>> to a number! (is method \"to_n\" well-defined?)";
+
+             $$_ = $tmp;
+           }
+        ) for @_;
     }
 
     sub _big2mpfr {
@@ -101,7 +128,7 @@ package Sidef::Types::Number::Number {
 
     sub _mpfr2big {
 
-        $PREC = $PREC->get_value if ref($PREC);
+        $PREC = "$PREC" if ref($PREC);
 
         if (Math::MPFR::Rmpfr_inf_p($_[0])) {
             if (Math::MPFR::Rmpfr_sgn($_[0]) > 0) {
@@ -188,7 +215,7 @@ package Sidef::Types::Number::Number {
         }
         elsif (($i = index($str, '.')) != -1) {
             my ($before, $after) = (substr($str, 0, $i), substr($str, $i + 1));
-            if ($after =~ tr/0// == length($after)) {
+            if (($after =~ tr/0//) == length($after)) {
                 return "$sign$before";
             }
             $sign . ("$before$after/1" =~ s/^0+//r) . ('0' x length($after));
@@ -198,102 +225,102 @@ package Sidef::Types::Number::Number {
         }
     }
 
-    sub get_value {
-        $GET_PERL_VALUE ? Math::GMPq::Rmpq_get_d(${$_[0]}) : do {
-            my $v = Math::GMPq::Rmpq_get_str(${$_[0]}, 10);
+    sub get_value { Math::GMPq::Rmpq_get_d(${$_[0]}) }
 
-            if (index($v, '/') != -1) {
-                my ($x) = @_;
-                $PREC = "$$PREC" if ref($PREC);
+    sub _as_str {
+        my $v = Math::GMPq::Rmpq_get_str(${$_[0]}, 10);
 
-                my $prec = CORE::int($PREC / 4);
-                my $sgn  = Math::GMPq::Rmpq_sgn($$x);
+        if (index($v, '/') != -1) {
+            my ($x) = @_;
+            $PREC = "$$PREC" if ref($PREC);
 
-                my $n = Math::GMPq::Rmpq_init();
-                Math::GMPq::Rmpq_set($n, $$x);
-                Math::GMPq::Rmpq_abs($n, $n) if $sgn < 0;
+            my $prec = CORE::int($PREC / 4);
+            my $sgn  = Math::GMPq::Rmpq_sgn($$x);
 
-                my $z = Math::GMPz::Rmpz_init();
-                Math::GMPz::Rmpz_ui_pow_ui($z, 10, CORE::abs($prec));
+            my $n = Math::GMPq::Rmpq_init();
+            Math::GMPq::Rmpq_set($n, $$x);
+            Math::GMPq::Rmpq_abs($n, $n) if $sgn < 0;
 
-                my $p = Math::GMPq::Rmpq_init();
-                Math::GMPq::Rmpq_set_z($p, $z);
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_ui_pow_ui($z, 10, CORE::abs($prec));
 
-                if ($prec < 0) {
-                    Math::GMPq::Rmpq_div($n, $n, $p);
-                }
-                else {
-                    Math::GMPq::Rmpq_mul($n, $n, $p);
-                }
+            my $p = Math::GMPq::Rmpq_init();
+            Math::GMPq::Rmpq_set_z($p, $z);
 
-                state $half = do {
-                    my $q = Math::GMPq::Rmpq_init();
-                    Math::GMPq::Rmpq_set_ui($q, 1, 2);
-                    $q;
-                };
-
-                Math::GMPq::Rmpq_add($n, $n, $half);
-                Math::GMPz::Rmpz_set_q($z, $n);
-
-                # Too much rounding... Give up and return an MPFR stringified number.
-                Math::GMPz::Rmpz_sgn($z) || do {
-                    my $mpfr = Math::MPFR::Rmpfr_init2($PREC);
-                    Math::MPFR::Rmpfr_set_q($mpfr, $$x, $ROUND);
-                    return Math::MPFR::Rmpfr_get_str($mpfr, 10, $prec, $ROUND);
-                };
-
-                if (Math::GMPz::Rmpz_odd_p($z) and Math::GMPq::Rmpq_integer_p($n)) {
-                    Math::GMPz::Rmpz_sub_ui($z, $z, 1);
-                }
-
-                Math::GMPq::Rmpq_set_z($n, $z);
-
-                if ($prec < 0) {
-                    Math::GMPq::Rmpq_mul($n, $n, $p);
-                }
-                else {
-                    Math::GMPq::Rmpq_div($n, $n, $p);
-                }
-
-                my $num = Math::GMPz::Rmpz_init();
-                my $den = Math::GMPz::Rmpz_init();
-
-                Math::GMPq::Rmpq_get_num($num, $n);
-                Math::GMPq::Rmpq_get_den($den, $n);
-
-                my @r;
-                my $c = 0;
-
-                while (1) {
-
-                    Math::GMPz::Rmpz_div($z, $num, $den);
-                    push @r, Math::GMPz::Rmpz_get_str($z, 10);
-
-                    Math::GMPz::Rmpz_mul($z, $z, $den);
-                    last if Math::GMPz::Rmpz_divisible_p($num, $den);
-                    Math::GMPz::Rmpz_sub($num, $num, $z);
-
-                    my $s = -1;
-                    while (Math::GMPz::Rmpz_cmp($den, $num) > 0) {
-                        last if !Math::GMPz::Rmpz_sgn($num);
-                        Math::GMPz::Rmpz_mul_ui($num, $num, 10);
-                        ++$s;
-                    }
-
-                    push(@r, '0' x $s) if ($s > 0);
-                }
-
-                ($sgn < 0 ? "-" : '') . ((shift(@r) . '.' . join('', @r)) =~ s/0+\z//r =~ s/\.\z//r);
+            if ($prec < 0) {
+                Math::GMPq::Rmpq_div($n, $n, $p);
             }
             else {
-                $v;
+                Math::GMPq::Rmpq_mul($n, $n, $p);
             }
-          }
+
+            state $half = do {
+                my $q = Math::GMPq::Rmpq_init();
+                Math::GMPq::Rmpq_set_ui($q, 1, 2);
+                $q;
+            };
+
+            Math::GMPq::Rmpq_add($n, $n, $half);
+            Math::GMPz::Rmpz_set_q($z, $n);
+
+            # Too much rounding... Give up and return an MPFR stringified number.
+            Math::GMPz::Rmpz_sgn($z) || do {
+                my $mpfr = Math::MPFR::Rmpfr_init2($PREC);
+                Math::MPFR::Rmpfr_set_q($mpfr, $$x, $ROUND);
+                return Math::MPFR::Rmpfr_get_str($mpfr, 10, $prec, $ROUND);
+            };
+
+            if (Math::GMPz::Rmpz_odd_p($z) and Math::GMPq::Rmpq_integer_p($n)) {
+                Math::GMPz::Rmpz_sub_ui($z, $z, 1);
+            }
+
+            Math::GMPq::Rmpq_set_z($n, $z);
+
+            if ($prec < 0) {
+                Math::GMPq::Rmpq_mul($n, $n, $p);
+            }
+            else {
+                Math::GMPq::Rmpq_div($n, $n, $p);
+            }
+
+            my $num = Math::GMPz::Rmpz_init();
+            my $den = Math::GMPz::Rmpz_init();
+
+            Math::GMPq::Rmpq_get_num($num, $n);
+            Math::GMPq::Rmpq_get_den($den, $n);
+
+            my @r;
+            my $c = 0;
+
+            while (1) {
+
+                Math::GMPz::Rmpz_div($z, $num, $den);
+                push @r, Math::GMPz::Rmpz_get_str($z, 10);
+
+                Math::GMPz::Rmpz_mul($z, $z, $den);
+                last if Math::GMPz::Rmpz_divisible_p($num, $den);
+                Math::GMPz::Rmpz_sub($num, $num, $z);
+
+                my $s = -1;
+                while (Math::GMPz::Rmpz_cmp($den, $num) > 0) {
+                    last if !Math::GMPz::Rmpz_sgn($num);
+                    Math::GMPz::Rmpz_mul_ui($num, $num, 10);
+                    ++$s;
+                }
+
+                push(@r, '0' x $s) if ($s > 0);
+            }
+
+            ($sgn < 0 ? "-" : '') . ((shift(@r) . '.' . join('', @r)) =~ s/0+\z//r =~ s/\.\z//r);
+        }
+        else {
+            $v;
+        }
     }
 
     sub base {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
 
         state $min = Math::GMPq->new(2);
         state $max = Math::GMPq->new(36);
@@ -358,14 +385,12 @@ package Sidef::Types::Number::Number {
     }
 
     sub phi {
-        state $one_f  = (Math::MPFR::Rmpfr_init_set_ui(1, $ROUND))[0];
-        state $two_f  = (Math::MPFR::Rmpfr_init_set_ui(2, $ROUND))[0];
-        state $five_f = (Math::MPFR::Rmpfr_init_set_ui(5, $ROUND))[0];
+        state $five4_f = (Math::MPFR::Rmpfr_init_set_str("1.25", 10, $ROUND))[0];
+        state $half_f  = (Math::MPFR::Rmpfr_init_set_str("0.5",  10, $ROUND))[0];
 
         my $phi = Math::MPFR::Rmpfr_init2($PREC);
-        Math::MPFR::Rmpfr_sqrt($phi, $five_f, $ROUND);
-        Math::MPFR::Rmpfr_add($phi, $phi, $one_f, $ROUND);
-        Math::MPFR::Rmpfr_div($phi, $phi, $two_f, $ROUND);
+        Math::MPFR::Rmpfr_sqrt($phi, $five4_f, $ROUND);
+        Math::MPFR::Rmpfr_add($phi, $phi, $half_f, $ROUND);
 
         _mpfr2big($phi);
     }
@@ -391,7 +416,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_add($r, $$x, $$y);
@@ -400,7 +425,7 @@ package Sidef::Types::Number::Number {
 
     sub iadd {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_add($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -419,7 +444,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_sub($r, $$x, $$y);
@@ -428,7 +453,7 @@ package Sidef::Types::Number::Number {
 
     sub isub {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_sub($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -448,7 +473,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_mul($r, $$x, $$y);
@@ -457,7 +482,7 @@ package Sidef::Types::Number::Number {
 
     sub imul {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_mul($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -476,7 +501,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (!Math::GMPq::Rmpq_sgn($$y)) {
             my $sign = Math::GMPq::Rmpq_sgn($$x);
@@ -490,7 +515,7 @@ package Sidef::Types::Number::Number {
 
     sub idiv {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
 
         my $r = _big2mpz($x);
         $y = _big2mpz($y);
@@ -514,9 +539,10 @@ package Sidef::Types::Number::Number {
     *negative = \&neg;
 
     sub abs {
-        my ($x) = @_;
+        my $q = ${$_[0]};
+        Math::GMPq::Rmpq_sgn($q) >= 0 and return ($_[0]);
         my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_abs($r, $$x);
+        Math::GMPq::Rmpq_abs($r, $q);
         bless \$r, __PACKAGE__;
     }
 
@@ -587,7 +613,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
         return $x->pow($y->inv);
     }
 
@@ -601,7 +627,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         my $r    = _big2mpz($x);
         my $root = CORE::int(Math::GMPq::Rmpq_get_d($$y));
@@ -639,7 +665,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_integer_p($$x) and Math::GMPq::Rmpq_integer_p($$y)) {
 
@@ -681,7 +707,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         state $ONE_Z = Math::GMPz::Rmpz_init_set_ui(1);
         my $pow = CORE::int(Math::GMPq::Rmpq_get_d($$y));
@@ -699,7 +725,7 @@ package Sidef::Types::Number::Number {
 
     sub fmod {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpfr($x);
         Math::MPFR::Rmpfr_fmod($r, $r, _big2mpfr($y), $ROUND);
         _mpfr2big($r);
@@ -724,7 +750,7 @@ package Sidef::Types::Number::Number {
                 return nan();
             }
 
-            _valid($y);
+            _valid(\$y);
             my $baseln = _big2mpfr($y);
             Math::MPFR::Rmpfr_log($baseln, $baseln, $ROUND);
             Math::MPFR::Rmpfr_div($r, $r, $baseln, $ROUND);
@@ -1048,7 +1074,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpfr($x);
         Math::MPFR::Rmpfr_atan2($r, $r, _big2mpfr($y), $ROUND);
         _mpfr2big($r);
@@ -1060,7 +1086,7 @@ package Sidef::Types::Number::Number {
 
     sub agm {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpfr($x);
         Math::MPFR::Rmpfr_agm($r, $r, _big2mpfr($y), $ROUND);
         _mpfr2big($r);
@@ -1068,7 +1094,7 @@ package Sidef::Types::Number::Number {
 
     sub hypot {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpfr($x);
         Math::MPFR::Rmpfr_hypot($r, $r, _big2mpfr($y), $ROUND);
         _mpfr2big($r);
@@ -1143,7 +1169,7 @@ package Sidef::Types::Number::Number {
 
     sub eq {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         if (Math::GMPq::Rmpq_equal($$x, $$y)) {
             (Sidef::Types::Bool::Bool::TRUE);
         }
@@ -1154,7 +1180,7 @@ package Sidef::Types::Number::Number {
 
     sub ne {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         if (Math::GMPq::Rmpq_equal($$x, $$y)) {
             (Sidef::Types::Bool::Bool::FALSE);
         }
@@ -1176,7 +1202,7 @@ package Sidef::Types::Number::Number {
             return;
         }
 
-        _valid($y);
+        _valid(\$y);
         my $cmp = Math::GMPq::Rmpq_cmp($$x, $$y);
         !$cmp ? (ZERO) : $cmp < 0 ? (MONE) : (ONE);
     }
@@ -1184,7 +1210,7 @@ package Sidef::Types::Number::Number {
     sub acmp {
         my ($x, $y) = @_;
 
-        _valid($y);
+        _valid(\$y);
 
         my $xn = $$x;
         my $yn = $$y;
@@ -1218,7 +1244,7 @@ package Sidef::Types::Number::Number {
             return (Sidef::Types::Bool::Bool::FALSE);
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_cmp($$x, $$y) > 0) {
             (Sidef::Types::Bool::Bool::TRUE);
@@ -1241,7 +1267,7 @@ package Sidef::Types::Number::Number {
             return (Sidef::Types::Bool::Bool::FALSE);
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_cmp($$x, $$y) >= 0) {
             (Sidef::Types::Bool::Bool::TRUE);
@@ -1264,7 +1290,7 @@ package Sidef::Types::Number::Number {
             return (Sidef::Types::Bool::Bool::FALSE);
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_cmp($$x, $$y) < 0) {
             (Sidef::Types::Bool::Bool::TRUE);
@@ -1287,7 +1313,7 @@ package Sidef::Types::Number::Number {
             return (Sidef::Types::Bool::Bool::FALSE);
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_cmp($$x, $$y) <= 0) {
             (Sidef::Types::Bool::Bool::TRUE);
@@ -1309,7 +1335,7 @@ package Sidef::Types::Number::Number {
 
     sub is_one {
         my ($x) = @_;
-        if (Math::GMPq::Rmpq_equal($$x, ${(ONE)})) {
+        if (Math::GMPq::Rmpq_equal($$x, $ONE)) {
             (Sidef::Types::Bool::Bool::TRUE);
         }
         else {
@@ -1319,7 +1345,7 @@ package Sidef::Types::Number::Number {
 
     sub is_mone {
         my ($x) = @_;
-        if (Math::GMPq::Rmpq_equal($$x, ${(MONE)})) {
+        if (Math::GMPq::Rmpq_equal($$x, $MONE)) {
             (Sidef::Types::Bool::Bool::TRUE);
         }
         else {
@@ -1418,7 +1444,7 @@ package Sidef::Types::Number::Number {
 
     sub is_div {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
 
         return Sidef::Types::Bool::Bool::FALSE
           if Math::GMPq::Rmpq_sgn($$y) == 0;
@@ -1448,7 +1474,7 @@ package Sidef::Types::Number::Number {
 
     sub divides {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
 
         return Sidef::Types::Bool::Bool::FALSE
           if Math::GMPq::Rmpq_sgn($$x) == 0;
@@ -1478,19 +1504,21 @@ package Sidef::Types::Number::Number {
 
     sub max {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         Math::GMPq::Rmpq_cmp($$x, $$y) > 0 ? $x : $y;
     }
 
     sub min {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         Math::GMPq::Rmpq_cmp($$x, $$y) < 0 ? $x : $y;
     }
 
     sub int {
+        my $q = ${$_[0]};
+        Math::GMPq::Rmpq_integer_p($q) && return ($_[0]);
         my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, ${$_[0]});
+        Math::GMPz::Rmpz_set_q($z, $q);
         _mpz2big($z);
     }
 
@@ -1502,11 +1530,19 @@ package Sidef::Types::Number::Number {
 
     sub rat { $_[0] }
 
+    *re   = \&rat;
+    *real = \&rat;
+
+    sub imag { ZERO }
+
+    *im        = \&imag;
+    *imaginary = \&imag;
+
     sub as_int {
         my ($x, $base) = @_;
 
         if (defined $base) {
-            _valid($base);
+            _valid(\$base);
             $base = CORE::int(Math::GMPq::Rmpq_get_d($$base));
             if ($base < 2 or $base > 36) {
                 die "[ERROR] base must be between 2 and 36, got $base\n";
@@ -1516,8 +1552,12 @@ package Sidef::Types::Number::Number {
             $base = 10;
         }
 
+        my $q = $$x;
+        Math::GMPq::Rmpq_integer_p($q)
+          && return (Sidef::Types::String::String->new(Math::GMPq::Rmpq_get_str($q, $base)));
+
         my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, $$x);
+        Math::GMPz::Rmpz_set_q($z, $q);
         Sidef::Types::String::String->new(Math::GMPz::Rmpz_get_str($z, $base));
     }
 
@@ -1525,7 +1565,7 @@ package Sidef::Types::Number::Number {
         my ($x, $prec) = @_;
 
         if (defined $prec) {
-            _valid($prec);
+            _valid(\$prec);
             $prec = Math::GMPq::Rmpq_get_d($$prec);
         }
         else {
@@ -1533,7 +1573,7 @@ package Sidef::Types::Number::Number {
         }
 
         local $Sidef::Types::Number::Number::PREC = 4 * $prec;
-        Sidef::Types::String::String->new($x->get_value);
+        Sidef::Types::String::String->new("$x");
     }
 
     sub as_rat {
@@ -1542,39 +1582,83 @@ package Sidef::Types::Number::Number {
 
     *dump = \&as_rat;
 
+    sub as_frac {
+        my $rat = Math::GMPq::Rmpq_get_str(${$_[0]}, 10);
+        Sidef::Types::String::String->new(index($rat, '/') != -1 ? $rat : "$rat/1");
+    }
+
     sub as_bin {
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, ${$_[0]});
-        Sidef::Types::String::String->new(Math::GMPz::Rmpz_get_str($z, 2));
+        my $q = ${$_[0]};
+        my $str =
+            Math::GMPq::Rmpq_integer_p($q)
+          ? Math::GMPq::Rmpq_get_str($q, 2)
+          : do {
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_set_q($z, $q);
+            Math::GMPz::Rmpz_get_str($z, 2);
+          };
+        Sidef::Types::String::String->new($str);
     }
 
     sub as_oct {
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, ${$_[0]});
-        Sidef::Types::String::String->new(Math::GMPz::Rmpz_get_str($z, 8));
+        my $q = ${$_[0]};
+        my $str =
+            Math::GMPq::Rmpq_integer_p($q)
+          ? Math::GMPq::Rmpq_get_str($q, 8)
+          : do {
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_set_q($z, $q);
+            Math::GMPz::Rmpz_get_str($z, 8);
+          };
+        Sidef::Types::String::String->new($str);
     }
 
     sub as_hex {
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, ${$_[0]});
-        Sidef::Types::String::String->new(Math::GMPz::Rmpz_get_str($z, 16));
+        my $q = ${$_[0]};
+        my $str =
+            Math::GMPq::Rmpq_integer_p($q)
+          ? Math::GMPq::Rmpq_get_str($q, 16)
+          : do {
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_set_q($z, $q);
+            Math::GMPz::Rmpz_get_str($z, 16);
+          };
+        Sidef::Types::String::String->new($str);
     }
 
     sub digits {
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, ${$_[0]});
-        Math::GMPz::Rmpz_abs($z, $z);
-        Sidef::Types::Array::Array->new([map { _new_uint($_) } split(//, Math::GMPz::Rmpz_get_str($z, 10))]);
+        my $q = ${$_[0]};
+
+        my $str =
+            Math::GMPq::Rmpq_integer_p($q)
+          ? Math::GMPq::Rmpq_get_str($q, 10)
+          : do {
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_set_q($z, $q);
+            Math::GMPz::Rmpz_get_str($z, 10);
+          };
+
+        $str =~ tr/-//d;
+        Sidef::Types::Array::Array->new([map { _new_uint($_) } split(//, $str)]);
     }
 
     sub digit {
         my ($x, $y) = @_;
-        _valid($y);
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_set_q($z, $$x);
-        Math::GMPz::Rmpz_abs($z, $z);
-        my $digit = (split(//, Math::GMPz::Rmpz_get_str($z, 10)))[Math::GMPq::Rmpq_get_d($$y)];
-        defined($digit) ? _new_uint($digit) : (MONE);
+        _valid(\$y);
+
+        my $q = $$x;
+        my $str =
+            Math::GMPq::Rmpq_integer_p($q)
+          ? Math::GMPq::Rmpq_get_str($q, 10)
+          : do {
+            my $z = Math::GMPz::Rmpz_init();
+            Math::GMPz::Rmpz_set_q($z, $q);
+            Math::GMPz::Rmpz_get_str($z, 10);
+          };
+
+        $str =~ tr/-//d;
+        my $digit = substr($str, Math::GMPq::Rmpq_get_d($$y), 1);
+        length($digit) ? _new_uint($digit) : nan();
     }
 
     sub length {
@@ -1582,7 +1666,7 @@ package Sidef::Types::Number::Number {
         Math::GMPz::Rmpz_set_q($z, ${$_[0]});
         Math::GMPz::Rmpz_abs($z, $z);
 
-        #_new_uint(Math::GMPz::Rmpz_sizeinbase($z, 10));        # turns out to be inexact
+        #_new_uint(Math::GMPz::Rmpz_sizeinbase($z, 10));        # turned out to be inexact
         _new_uint(Math::GMPz::Rmpz_snprintf(my $buf, 0, "%Zd", $z, 0));
     }
 
@@ -1626,14 +1710,14 @@ package Sidef::Types::Number::Number {
     sub inc {
         my ($x) = @_;
         my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_add($r, $$x, ${(ONE)});
+        Math::GMPq::Rmpq_add($r, $$x, $ONE);
         bless \$r, __PACKAGE__;
     }
 
     sub dec {
         my ($x) = @_;
         my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_sub($r, $$x, ${(ONE)});
+        Math::GMPq::Rmpq_sub($r, $$x, $ONE);
         bless \$r, __PACKAGE__;
     }
 
@@ -1651,7 +1735,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         if (Math::GMPq::Rmpq_integer_p($$x) and Math::GMPq::Rmpq_integer_p($$y)) {
 
@@ -1694,7 +1778,7 @@ package Sidef::Types::Number::Number {
             return nan();
         }
 
-        _valid($y);
+        _valid(\$y);
 
         my $yz     = _big2mpz($y);
         my $sign_y = Math::GMPz::Rmpz_sgn($yz);
@@ -1713,7 +1797,7 @@ package Sidef::Types::Number::Number {
 
     sub modpow {
         my ($x, $y, $z) = @_;
-        _valid($y, $z);
+        _valid(\$y, \$z);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_powm($r, $r, _big2mpz($y), _big2mpz($z));
         _mpz2big($r);
@@ -1723,7 +1807,7 @@ package Sidef::Types::Number::Number {
 
     sub modinv {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_invert($r, $r, _big2mpz($y)) || return nan();
         _mpz2big($r);
@@ -1734,7 +1818,7 @@ package Sidef::Types::Number::Number {
     sub divmod {
         my ($x, $y) = @_;
 
-        _valid($y);
+        _valid(\$y);
 
         my $r1 = _big2mpz($x);
         my $r2 = _big2mpz($y);
@@ -1747,7 +1831,7 @@ package Sidef::Types::Number::Number {
 
     sub and {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_and($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -1755,7 +1839,7 @@ package Sidef::Types::Number::Number {
 
     sub or {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_ior($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -1763,7 +1847,7 @@ package Sidef::Types::Number::Number {
 
     sub xor {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_xor($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -1816,7 +1900,7 @@ package Sidef::Types::Number::Number {
 
     sub binomial {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_bin_si($r, $r, CORE::int(Math::GMPq::Rmpq_get_d($$y)));
         _mpz2big($r);
@@ -1826,19 +1910,19 @@ package Sidef::Types::Number::Number {
 
     sub legendre {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         _new_int(Math::GMPz::Rmpz_legendre(_big2mpz($x), _big2mpz($y)));
     }
 
     sub jacobi {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         _new_int(Math::GMPz::Rmpz_jacobi(_big2mpz($x), _big2mpz($y)));
     }
 
     sub kronecker {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         _new_int(Math::GMPz::Rmpz_kronecker(_big2mpz($x), _big2mpz($y)));
     }
 
@@ -1852,7 +1936,7 @@ package Sidef::Types::Number::Number {
 
     sub gcd {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_gcd($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -1860,7 +1944,7 @@ package Sidef::Types::Number::Number {
 
     sub lcm {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         Math::GMPz::Rmpz_lcm($r, $r, _big2mpz($y));
         _mpz2big($r);
@@ -1873,7 +1957,7 @@ package Sidef::Types::Number::Number {
         if (
             Math::GMPz::Rmpz_probab_prime_p(_big2mpz($x),
                                             defined($k)
-                                            ? do { _valid($k); CORE::int Math::GMPq::Rmpq_get_d($$k) }
+                                            ? do { _valid(\$k); CORE::int Math::GMPq::Rmpq_get_d($$k) }
                                             : 7) > 0
           ) {
             (Sidef::Types::Bool::Bool::TRUE);
@@ -1952,7 +2036,7 @@ package Sidef::Types::Number::Number {
     sub next_pow {
         my ($x, $y) = @_;
 
-        _valid($y);
+        _valid(\$y);
 
         my $f = Math::MPFR::Rmpfr_init2($PREC);
         Math::MPFR::Rmpfr_set_z($f, _big2mpz($x), $PREC);
@@ -1976,7 +2060,7 @@ package Sidef::Types::Number::Number {
 
     sub shift_left {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         my $i = CORE::int(Math::GMPq::Rmpq_get_d($$y));
         if ($i < 0) {
@@ -1990,7 +2074,7 @@ package Sidef::Types::Number::Number {
 
     sub shift_right {
         my ($x, $y) = @_;
-        _valid($y);
+        _valid(\$y);
         my $r = _big2mpz($x);
         my $i = CORE::int(Math::GMPq::Rmpq_get_d($$y));
         if ($i < 0) {
@@ -2064,67 +2148,9 @@ package Sidef::Types::Number::Number {
         $i->mul($x);
     }
 
-    sub array_to {
-        my ($from, $to, $step) = @_;
-
-        if (!defined($step)) {
-            _valid($to);
-            $step = (ONE);
-        }
-        else {
-            _valid($to, $step);
-        }
-
-        $from = $$from;
-        $to   = $$to;
-        $step = $$step;
-
-        my $acc = Math::GMPq::Rmpq_init();
-        my @array;
-        for (Math::GMPq::Rmpq_set($acc, $from) ;
-             Math::GMPq::Rmpq_cmp($acc, $to) <= 0 ; Math::GMPq::Rmpq_add($acc, $acc, $step)) {
-            my $copy = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set($copy, $acc);
-            push @array, bless(\$copy, __PACKAGE__);
-        }
-
-        Sidef::Types::Array::Array->new(\@array);
-    }
-
-    *arr_to = \&array_to;
-
-    sub array_downto {
-        my ($from, $to, $step) = @_;
-
-        if (!defined($step)) {
-            _valid($to);
-            $step = (ONE);
-        }
-        else {
-            _valid($to, $step);
-        }
-
-        $from = $$from;
-        $to   = $$to;
-        $step = $$step;
-
-        my $acc = Math::GMPq::Rmpq_init();
-        my @array;
-        for (Math::GMPq::Rmpq_set($acc, $from) ;
-             Math::GMPq::Rmpq_cmp($acc, $to) >= 0 ; Math::GMPq::Rmpq_sub($acc, $acc, $step)) {
-            my $copy = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set($copy, $acc);
-            push @array, bless(\$copy, __PACKAGE__);
-        }
-
-        Sidef::Types::Array::Array->new(\@array);
-    }
-
-    *arr_downto = \&array_downto;
-
     sub round {
         my ($x, $prec) = @_;
-        _valid($prec);
+        _valid(\$prec);
 
         my $nth = -CORE::int(Math::GMPq::Rmpq_get_d($$prec));
         my $sgn = Math::GMPq::Rmpq_sgn($$x);
@@ -2179,44 +2205,38 @@ package Sidef::Types::Number::Number {
 
     sub to {
         my ($from, $to, $step) = @_;
-
-        if (ref($to) eq 'Sidef::Types::Number::Inf' or ref($to) eq 'Sidef::Types::Number::Ninf') {
-            _valid($step) if defined($step);
-        }
-        elsif (defined $step) {
-            _valid($to, $step);
-        }
-        else {
-            _valid($to);
-        }
-
-        Sidef::Types::Range::RangeNumber->__new__(
-                                                  from => $$from,
-                                                  to   => $$to,
-                                                  step => (defined($step) ? $$step : ${(ONE)}),
-                                                 );
+        Sidef::Types::Range::RangeNumber->new($from, $to, $step // ONE,);
     }
 
     *upto = \&to;
 
     sub downto {
         my ($from, $to, $step) = @_;
+        Sidef::Types::Range::RangeNumber->new($from, $to, defined($step) ? $step->neg : MONE);
+    }
 
-        if (ref($to) eq 'Sidef::Types::Number::Inf' or ref($to) eq 'Sidef::Types::Number::Ninf') {
-            _valid($step) if defined($step);
-        }
-        elsif (defined $step) {
-            _valid($to, $step);
-        }
-        else {
-            _valid($to);
-        }
+    sub xto {
+        my ($from, $to, $step) = @_;
 
-        Sidef::Types::Range::RangeNumber->__new__(
-                                                  from => $$from,
-                                                  to   => $$to,
-                                                  step => (defined($step) ? -$$step : ${(MONE)}),
-                                                 );
+        $to =
+          defined($step)
+          ? $to->sub($step)
+          : $to->dec;
+
+        Sidef::Types::Range::RangeNumber->new($from, $to, $step // ONE);
+    }
+
+    *xupto = \&xto;
+
+    sub xdownto {
+        my ($from, $to, $step) = @_;
+
+        $from =
+          defined($step)
+          ? $from->sub($step)
+          : $from->dec;
+
+        Sidef::Types::Range::RangeNumber->new($from, $to, defined($step) ? $step->neg : MONE);
     }
 
     sub range {
@@ -2240,7 +2260,14 @@ package Sidef::Types::Number::Number {
         Math::MPFR::Rmpfr_get_q($q, $rand);
 
         if (defined $y) {
-            _valid($y);
+
+            if (   ref($y) eq 'Sidef::Types::Number::Inf'
+                or ref($y) eq 'Sidef::Types::Number::Ninf'
+                or ref($y) eq 'Sidef::Types::Number::Nan') {
+                return $y;
+            }
+
+            _valid(\$y);
 
             my $diff = Math::GMPq::Rmpq_init();
             Math::GMPq::Rmpq_sub($diff, $$y, $$x);
@@ -2263,7 +2290,14 @@ package Sidef::Types::Number::Number {
         $x = _big2mpz($x);
 
         if (defined($y)) {
-            _valid($y);
+
+            if (   ref($y) eq 'Sidef::Types::Number::Inf'
+                or ref($y) eq 'Sidef::Types::Number::Ninf'
+                or ref($y) eq 'Sidef::Types::Number::Nan') {
+                return $y;
+            }
+
+            _valid(\$y);
 
             my $rand = _big2mpz($y);
             my $cmp = Math::GMPz::Rmpz_cmp($rand, $x);
@@ -2289,15 +2323,13 @@ package Sidef::Types::Number::Number {
         }
     }
 
-    *rand_int = \&irand;
-
     sub of {
         my ($x, $obj) = @_;
 
         if (ref($obj) eq 'Sidef::Types::Block::Block') {
 
             my @array;
-            my $num = $$x;
+            my $num = _big2mpz($x);
 
             for (my $i = Math::GMPz::Rmpz_init_set_ui(1) ;
                  Math::GMPz::Rmpz_cmp($i, $num) <= 0 ;
@@ -2333,7 +2365,7 @@ package Sidef::Types::Number::Number {
     sub times {
         my ($num, $block) = @_;
 
-        $num = $$num;
+        $num = _big2mpz($num);
 
         for (my $i = Math::GMPz::Rmpz_init_set_ui(1) ;
              Math::GMPz::Rmpz_cmp($i, $num) <= 0 ; Math::GMPz::Rmpz_add_ui($i, $i, 1)) {
@@ -2348,7 +2380,7 @@ package Sidef::Types::Number::Number {
     sub itimes {
         my ($num, $block) = @_;
 
-        $num = $$num;
+        $num = _big2mpz($num);
 
         for (my $i = Math::GMPz::Rmpz_init_set_ui(0) ; Math::GMPz::Rmpz_cmp($i, $num) < 0 ; Math::GMPz::Rmpz_add_ui($i, $i, 1))
         {
@@ -2363,7 +2395,7 @@ package Sidef::Types::Number::Number {
     sub commify {
         my ($self) = @_;
 
-        my $n = $self->get_value;
+        my $n = "$self";
 
         my $x   = $n;
         my $neg = $n =~ s{^-}{};
@@ -2488,9 +2520,9 @@ package Sidef::Types::Number::Number {
         *{__PACKAGE__ . '::' . '=='}  = \&eq;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '≠'} = \&ne;
-        *{__PACKAGE__ . '::' . '..'}  = \&array_to;
-        *{__PACKAGE__ . '::' . '..^'} = \&to;
-        *{__PACKAGE__ . '::' . '^..'} = \&downto;
+        *{__PACKAGE__ . '::' . '..'}  = \&to;
+        *{__PACKAGE__ . '::' . '..^'} = \&xto;
+        *{__PACKAGE__ . '::' . '^..'} = \&xdownto;
         *{__PACKAGE__ . '::' . '!'}   = \&factorial;
         *{__PACKAGE__ . '::' . '%%'}  = \&is_div;
         *{__PACKAGE__ . '::' . '>>'}  = \&shift_right;
